import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
/**Basisklasse fr alle Objekte im Programm EFeld
 * @author Oliver Barta
 * @version 1.0
 */
public abstract class Charge extends BranchGroup{
    protected EFeld efeld;
    /**Position des Objekts*/
    public Vektor3D pos;
    protected double Q_As;
    protected double breite;
    protected double hohe;
    protected double tiefe;
    protected String specification;
    protected Color3f originalcolor;
    protected Color3f color;
    protected boolean originalemitting;
    protected boolean emitting;
    protected TransformGroup transform;
    protected Transform3D transformrot;
    protected Vector3d drehung;
    protected Vector3d drehungoben;
    protected Appearance appearance;
    /**Erzeugt ein neues Objekt.
     * @param efeld EFeld
     * @param QinAs Ladung in As
     * @param x x-Koordinate
     * @param y y-Koordinate
     * @param z z-Koordinate
     * @param breite Breite des Objekts
     * @param hohe Hhe des Objekts
     * @param tiefe Tiefe des Objekts
     * @param color Farbe, die verwendet werden soll, wenn die Ladung null ist
     * @param emitting zeigt an, ob das Objekt selber Elektronen aussenden soll
     * @param ausrichtung Ausrichtung des Objekts
     * @param ausrichtungoben Vektor, der vom Objekt aus gesehen nach oben zeigt
     */
    public Charge(EFeld efeld, double QinAs, double x, double y, double z, double breite, double hohe, double tiefe, Color3f color, boolean emitting, Vector3d ausrichtung, Vector3d ausrichtungoben){
        this.efeld = efeld;
        this.setCapability(BranchGroup.ALLOW_DETACH);
        this.pos = new Vektor3D(x, y, z);
        this.originalcolor = color;
        this.specification = "Charge";
        this.originalemitting = emitting;
        this.setCharge(QinAs);
        if(ausrichtung != null)
            this.drehung = ausrichtung;
        else
            this.drehung = new Vector3d(0,0,-1);
        if(ausrichtungoben != null)
            this.drehungoben = ausrichtungoben;
        else
            this.drehungoben = new Vector3d(0,1,0);
        this.transformrot = new Transform3D();
        this.transformrot.lookAt(new Point3d(0,0,0), new Point3d(this.drehung.x, this.drehung.y, this.drehung.z), this.drehungoben);
        this.transformrot.invert();
        this.setScale(new Vector3d(breite, hohe, tiefe));
        this.transform = new TransformGroup();
        this.transform.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        this.transform.addChild(this.createVisibleObjekt());
        this.addChild(this.transform);
        this.refreshTransform();
    }
    /**Setzt die Ladung.
     * @param QinAs neue Ladung
     * @return true
     */
    public boolean setCharge(double QinAs){
        this.Q_As = QinAs;
        if(QinAs < 0)
            this.setColor(new Color3f(0.3f,0.3f,1f));
        else if(QinAs > 0)
            this.setColor(new Color3f(1f,0.2f,0.2f));
        else this.setColor(this.originalcolor);
        this.setEmitting(this.originalemitting);
        return true;
    }
    /**Gibt die aktuelle Ladung zurck.
     * @return Ladung in As
     */
    public double getCharge(){
        return this.Q_As;
    }
    /**Aktiviert bzw. Deaktiviert das Aussenden von Elektronen.
     * Elektronen werden nur ausgesendet wenn zustzlich die Ladung <= 0 ist
     * @param emitting ein oder aus
     */
    public void setEmitting(boolean emitting){
        this.originalemitting = emitting;
        this.emitting = this.originalemitting && (this.Q_As <= 0);
    }
    /**Zeigt an, ob das Objekt selber Elektronen aussenden soll.
     * @return ein oder aus
     * @see #setEmitting
     */
    public boolean getEmitting(){
        return this.originalemitting;
    }
    /**Setzt die Gre des Objekts.
     * @param scale Gren in x-, y- und z-Richtung
     * @return {true, true, true}
     */
    public boolean[] setScale(Vector3d scale){
        this.breite = scale.x;
        this.hohe = scale.y;
        this.tiefe = scale.z;
        this.transformrot.setScale(scale);
        boolean[] suc = {true, true, true};
        return suc;
    }
    /**Gibt die Gre zurck.
     * @return Gre in x-, y- und z-Richtung
     */
    public Vector3d getScale(){
        return new Vector3d(this.breite,this.hohe,this.tiefe);
    }
    /**Gibt das elektrische Feld, das von diesem Objekt erzeugt wird, im Punkt position an.
     * @param position Punkt in dem das Feld bestimmt werden soll
     * @return Feldvektor
     */
    public abstract Vektor3D getField(Vektor3D position);
    /**Gibt Daten wie Spezifikation, Position und Ladung in Stringform zurck.
     * @return Daten
     */
    public String toString(){
        return(this.specification+" at ["+this.pos.x+"/"+this.pos.y+"/"+this.pos.z+"] with"+" Q = "+this.Q_As+"As; ");
    }
    /**Gibt die Spezifikation zurck.
     * @return Spezifikation
     */
    public String getSpecification(){
        return this.specification;
    }
    /**Methode die Elektronen in zugehriges EFeld Programm hinzufgt.*/
    public void addElectrons(){}
    /**Fgt einen Vektor zur aktuellen Position hinzu.
     * @param move hinzuzufgender Vektor
     */
    public void addToPosition(Vector3d move){
        this.pos.x += move.x;
        this.pos.y += move.y;
        this.pos.z += move.z;
    }
    /**Aktualisiert die Position der 3D-Darstellung.*/
    public void refreshTransform(){
        Transform3D transform = new Transform3D();
        transform.set(new Vector3d(this.pos.x,this.pos.y,this.pos.z));
        transform.mul(this.transformrot);
        this.transform.setTransform(transform);
    }
    /**Setzt die Rotation auf null.*/
    public void clearRotation(){
        Transform3D transform = new Transform3D();
        this.drehung = new Vector3d(0,0,-1);
        this.drehungoben = new Vector3d(0,1,0);
        transform.lookAt(new Point3d(0,0,0), new Point3d(this.drehung.x, this.drehung.y, this.drehung.z), this.drehungoben);
        transform.invert();
        Vector3d scale = new Vector3d();
        this.transformrot.getScale(scale);
        this.transformrot.set(transform);
        this.transformrot.setScale(scale);
        this.refreshTransform();
    }
    /**Rotiert das Objekt um die x-Achse.
     * @param rad Anzahl der Radianten um die rotiert werden soll
     */
    public void rotX(double rad){
        Transform3D transform = new Transform3D();
        transform.rotX(rad);
        transform.transform(this.drehung);
        transform.transform(this.drehungoben);
        transform.lookAt(new Point3d(0,0,0), new Point3d(this.drehung.x, this.drehung.y, this.drehung.z), this.drehungoben);
        transform.invert();
        Vector3d scale = new Vector3d();
        this.transformrot.getScale(scale);
        this.transformrot.set(transform);
        this.transformrot.setScale(scale);
        this.refreshTransform();
    }
    /**Rotiert das Objekt um die y-Achse.
     * @param rad Anzahl der Radianten um die rotiert werden soll
     */
    public void rotY(double rad){
        Transform3D transform = new Transform3D();
        transform.rotY(rad);
        transform.transform(this.drehung);
        transform.transform(this.drehungoben);
        transform.lookAt(new Point3d(0,0,0), new Point3d(this.drehung.x, this.drehung.y, this.drehung.z), this.drehungoben);
        transform.invert();
        Vector3d scale = new Vector3d();
        this.transformrot.getScale(scale);
        this.transformrot.set(transform);
        this.transformrot.setScale(scale);
        this.refreshTransform();
    }
    /**Rotiert das Objekt um die z-Achse.
     * @param rad Anzahl der Radianten um die rotiert werden soll
     */
    public void rotZ(double rad){
        Transform3D transform = new Transform3D();
        transform.rotZ(rad);
        transform.transform(this.drehung);
        transform.transform(this.drehungoben);
        transform.lookAt(new Point3d(0,0,0), new Point3d(this.drehung.x, this.drehung.y, this.drehung.z), this.drehungoben);
        transform.invert();
        Vector3d scale = new Vector3d();
        this.transformrot.getScale(scale);
        this.transformrot.set(transform);
        this.transformrot.setScale(scale);
        this.refreshTransform();
    }
    /**Fgt einen Transform zum Aktuellen hinzu.
     * @param transform hinzuzufgender Transform
     */
    public void mulTransform(Transform3D transform){
        transform.transform(this.drehung);
        transform.transform(this.drehungoben);
        transform.lookAt(new Point3d(0,0,0), new Point3d(this.drehung.x, this.drehung.y, this.drehung.z), this.drehungoben);
        transform.invert();
        Vector3d scale = new Vector3d();
        this.transformrot.getScale(scale);
        this.transformrot.set(transform);
        this.transformrot.setScale(scale);
        this.refreshTransform();
    }
    /**Methode zur Vorbereitung der Bewegung, aber ohne das Objekt zu bewegen.
     * @param charges alle Objekte in der aktuellen Simulation. Elektronen bekommen alle statischen Ladungen bergeben; statische Ladungen bekommen alle Elektronen bergeben (Grund: Einsparen von Rechenkapazitt)
     * @return Vector Objekte die gelscht werden sollen
     */
    public abstract Vector prepareMove(Vector charges, double timestep);
    /**Methode zum Bewegen des Objekts. Wird nur dann aufgerufen, wenn Klasse Elektron.class (Grund: Einsparen von Rechenkapazitt).
     * @param timestep Zeitintervall um das das Objekt bewegt werden soll
     */
    public void move(double timestep){};
    /**Erzeugt das sichtbare Objekt. Wird bei der Initialisierung aufgerufen.
     * @return Sichtbares Objekt
     */
    protected abstract BranchGroup createVisibleObjekt();
    /**Setzt die Farbe, die verwendet werden soll, wenn die Ladung <= 0 ist.
     * @param color Farbe
     */
    public void setColor(Color3f color){
        this.color = color;
        if(this.appearance == null){
            this.appearance = new Appearance();
            this.appearance.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
        }
        this.appearance.setMaterial(new Material(color, new Color3f(), new Color3f(), color, 1f));
    }
    /**Gibt die Farbe zurck, die verwendet werden soll, wenn die Ladung <= 0 ist.
     * @return Farbe
     */
    public Color3f getColor(){
        return this.originalcolor;
    }
    /**Gibt Anfangspunkte von Feldlinien fr dieses Objekt zurck.
     * @param count ungefhre Anzahl
     * @return Anfangspunkte
     */
    public abstract double[][] getFieldlineStartingPoints(int count);
    /**Zeigt an ob ein kugelfrmiges Objekt mit diesem kollidiert ist.
     * @param position Position des anderen Objekts
     * @param radius Radius des anderen Objekts
     * @param precision wird nur beim Zeichnen der Feldlinien und nur von PlateCharge ausgewertet
     * @return Kollision oder keine Kollision
     */
    public abstract boolean collided(Vektor3D position, double radius, int precision);
    /**Gibt alle Informationen, die zur Speicherung dieses Objekts ntig sind, zurck.
     * @return Informationen
     */
    public abstract String getSaveString();
}
