// stopView, renderOnce evtl. streichen
import com.sun.org.apache.bcel.internal.generic.ArrayInstruction;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
/**Hauptfenster von EFeld. Dieses Programm dient zur dreidimensionalen Simulation elektrischer Felder. Es knnen unterschiedliche statische Ladungsverteilungen eingestellt werden und die Bewegung von Elektronen in diesen Feldern simuliert werden.
 * @author Oliver Barta
 * @version 1.01
 */
public class EFeld extends Frame implements WindowListener, ActionListener, Runnable, ItemListener, KeyListener, MouseWheelListener, MouseMotionListener, MouseListener, DropTargetListener{
    protected String country = "US";
    private int hauptfensterbreite = 640;
    private int hauptfensterhohe = 480;
    private int hauptfensterxpos = 0;
    private int hauptfensterypos = 0;
    protected EFeldMenu efeldmenu;
    private TimeFrame timeframe;
    protected int timeframebreite = 200;
    protected int timeframehohe = 100;
    protected int timeframexpos = 0;
    protected int timeframeypos = 0;
    protected boolean timeframevisible = true;
    private ObjectManager objectframe;
    protected int objectframebreite = 200;
    protected int objectframehohe = 100;
    protected int objectframexpos = 0;
    protected int objectframeypos = 0;
    protected boolean objectframevisible = false;
    protected double time_nano = 0;
    protected double timestep_nano = 25;
    protected int precision = 5;
    protected Vector ladungst;
    protected Vector ladungel;
    protected boolean pause = false;
    protected long bildrate[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    protected double bildzahl = 0;
    private double delay = 40;
    protected BranchGroup objectgroupst;
    protected BranchGroup objectgroupel;
    protected BranchGroup objectgroupfl;
    private TransformGroup viewtransmitte;
    private double[] transmitte = {0,0,0};
    private TransformGroup viewtransdist;
    private int distance = 20;
    private TransformGroup viewtransrot;
    private Transform3D viewrot;
    private View view;
    protected double computingarea = 10;
    protected boolean boundingvisible = false;
    protected boolean fieldlinesvisible = false;
    protected LineArray boundingbox;
    private int[] dragpos = new int[2];
    protected int selectedobject = -1;
    protected Vektor3D ortsfaktor = new Vektor3D();
    private double rotx = 0;
    private double roty = 0;
    private boolean[] arrowdown = {false,false,false,false};
    private String stsave;
    private String stload;
    private String[][] scenes = new String[10][0];
    protected boolean autochange = false;
    private int wait = 0;
    /**Startet EFeld.
     * @param args Die Datei, deren Pfad durch den ersten Parameter spezifiziert wird, wird als Szenario geladen. Sollte kein Parameter angegeben werden wird ein Standardszenario geladen.
     */
    public static void main(String[] args){
        if((args != null) && (args.length > 0))
            new EFeld(args[0]);
        else
            new EFeld(null);
    }
    /**Erzeugt eine neue Instanz dieser Klasse
     * @param path Pfad der Datei die als Szenario geladen werden soll. Wert null fr Standardszenario.
     */
    public EFeld(String path){
        
        this.ladungst = new Vector();
        this.ladungel = new Vector();
        this.loadOption();

        this.addWindowListener(this);
        this.addKeyListener(this);
        this.addMouseWheelListener(this);
        this.efeldmenu = new EFeldMenu(this);
        this.setMenuBar(this.efeldmenu);
        this.timeframe = new TimeFrame(this);
        this.objectframe = new ObjectManager(this);

        DropTarget droptarget = new DropTarget(this, this);

        VirtualUniverse virtualuniverse = new VirtualUniverse();
        javax.media.j3d.Locale locale = new javax.media.j3d.Locale(virtualuniverse);

        //BranchGroups fr Objekte
        this.objectgroupst = new BranchGroup();
        this.objectgroupst.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
        this.objectgroupst.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
        this.objectgroupst.compile();
        locale.addBranchGraph(this.objectgroupst);

        this.objectgroupfl = new BranchGroup();
        this.objectgroupfl.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
        this.objectgroupfl.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
        this.objectgroupfl.compile();
        locale.addBranchGraph(this.objectgroupfl);

        this.objectgroupel = new BranchGroup();
        this.objectgroupel.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
        this.objectgroupel.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
        this.objectgroupel.compile();
        locale.addBranchGraph(this.objectgroupel);

        //BranchGroup fr BoundingGox
        this.boundingbox = new LineArray(64, LineArray.COORDINATES);
        this.boundingbox.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
        Appearance lineappearance = new Appearance();
        lineappearance.setLineAttributes(new LineAttributes(1f, LineAttributes.PATTERN_SOLID, false));
        lineappearance.setColoringAttributes(new ColoringAttributes(1f, 1f, 0f, ColoringAttributes.SHADE_FLAT));
        BranchGroup boundingboxgroup = new BranchGroup();
        boundingboxgroup.addChild(new Shape3D(this.boundingbox, lineappearance));
        boundingboxgroup.compile();
        locale.addBranchGraph(boundingboxgroup);

        //BranchGroup fr Pointlight
        BranchGroup lightgroup = new BranchGroup();
        PointLight pointlight1 = new PointLight(new Color3f(1f,1f,1f), new Point3f(10f, 5f, 15f), new Point3f(1f, 0.03f, 0f));
        pointlight1.setInfluencingBounds(new BoundingSphere(new Point3d(0.,0.,0.), 1000.));
        lightgroup.addChild(pointlight1);
        lightgroup.compile();
        locale.addBranchGraph(lightgroup);

        //BranchGroup fr View
        BranchGroup viewgroup = new BranchGroup();
        this.viewtransrot = new TransformGroup();
        viewtransrot.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

        this.viewtransdist = new TransformGroup();
        this.viewtransdist.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

        Background background = new Background(0f,0f,0f);
        background.setApplicationBounds(new BoundingSphere(new Point3d(0.,0.,0.), 1.));
        this.viewtransdist.addChild(background);
        AmbientLight ambientlight = new AmbientLight(new Color3f(0.7f,0.7f,0.7f));
        ambientlight.setInfluencingBounds(new BoundingSphere(new Point3d(0.,0.,0.), 1000.));
        this.viewtransdist.addChild(ambientlight);

        this.viewtransmitte = new TransformGroup();
        this.viewtransmitte.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        Transform3D trans3mitte = new Transform3D();
        trans3mitte.set(new Vector3d(this.transmitte[0], this.transmitte[1], this.transmitte[2]));
        this.viewtransmitte.setTransform(trans3mitte);
        this.viewtransmitte.addChild(this.viewtransrot);
        
        this.viewtransrot.addChild(this.viewtransdist);
        viewgroup.addChild(this.viewtransmitte);

        //ViewPlatform in die BranchGroup fr View hinzufgen
        ViewPlatform viewplatform = new ViewPlatform();
        viewplatform.setActivationRadius(620f);
        this.viewtransdist.addChild(viewplatform);

        PhysicalBody physicalbody = new PhysicalBody();

        PhysicalEnvironment physicalenvironment = new PhysicalEnvironment();

        Canvas3D canvas3d = new Canvas3D(GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getBestConfiguration(new GraphicsConfigTemplate3D()));
        canvas3d.setStereoEnable(canvas3d.getStereoAvailable());
        canvas3d.setBackground(new Color(0,0,0));
        canvas3d.addMouseListener(this);
        canvas3d.addMouseMotionListener(this);
        canvas3d.addKeyListener(this);
        canvas3d.addMouseWheelListener(this);

        //View Objekt erzeugen und verlinken
        this.view = new View();
        this.view.setBackClipDistance(50);
        this.view.attachViewPlatform(viewplatform);
        this.view.setPhysicalBody(physicalbody);
        this.view.setPhysicalEnvironment(physicalenvironment);
//        this.view.stopView();
        this.view.addCanvas3D(canvas3d);

        this.add(canvas3d);

        viewgroup.compile();
        locale.addBranchGraph(viewgroup);

        this.applyOption();

        if(path == null){
            this.addCharge(new PointCharge(this, -1E-7, 0, 0, 0, 1, new Color3f(0f,1f,0f), true));
        }else
            this.loadScene(path, false);

        this.setVisible(true);
        Thread thread = new Thread(this);
        thread.start();
    }
    /**Threadmethode, die fr die periodische Aktualisierung aller Fenster verantwortlich ist.*/
    public void run(){
        while(true)
            try{
                if(this.pause){
                    Thread.sleep(200);
                }else{
                    if(this.autochange){
                        if(wait <= 0){
                            int number = (int)(Math.random()*10);
                            int count = 0;
                            while((this.scenes[number].length < 2) && (count < 10)){
                                number++;
                                if(number > 9)
                                    number = 0;
                                count++;
                            }
                            if(count < 10)
                                this.shortcutKey(false, false, number);
                            this.wait = 800+(int)(Math.random()*600);
                        }else
                            this.wait--;
                    }
                    double timestep_nano = this.timestep_nano;
                    this.time_nano += timestep_nano;
                    this.calculate(timestep_nano/1.E9, this.precision);
                    Thread.sleep((long)this.delay);
                    if(++this.bildrate[0] >= this.bildrate.length)
                        this.bildrate[0] = 1;
                    this.bildrate[(int)(this.bildrate[0])] = System.currentTimeMillis();
                    this.bildzahl = 0;
                    if(this.bildrate[0]+1 >= this.bildrate.length)
                        bildzahl = ((double)this.bildrate.length-2.)/((double)this.bildrate[(int)this.bildrate[0]]-(double)this.bildrate[1])*1000;
                    else
                        bildzahl = ((double)this.bildrate.length-2.)/((double)this.bildrate[(int)this.bildrate[0]]-(double)this.bildrate[(int)this.bildrate[0]+1])*1000;
                    if(this.bildzahl > 21)
                        this.delay += 1E-2;
                    else
                        if(this.delay > 1)
                            this.delay -= 1E-2;
                    if((this.rotx != 0) || (this.roty != 0)){
                        Transform3D trans = new Transform3D();
                        trans.rotX(this.rotx);
                        this.viewrot.mul(trans);
                        trans = new Transform3D();
                        trans.rotY(this.roty);
                        this.viewrot.mul(trans);
                        this.applyView();
                    }
//                    this.refresh();
                    if(this.timeframe.isVisible())
                        this.timeframe.refresh();
                }
            }catch(InterruptedException ire){}
    }
/*    public void refresh(){
//        this.view.renderOnce();
    }*/
    /**Aktualisiert alle Fenster*/
    public void refreshAll(){
//        this.refresh();
        this.applyFieldlines();
        this.timeframe.refresh();
        this.timeframe.refreshValues();
        this.objectframe.refresh();
        this.objectframe.refreshValues();
    }
    /**Fgt alle Objekte des Vectors hinzu.
     * @param charges Vector mit allen (Charge)Objekten, die hinzugefgt werden sollen
     */
    public void addCharges(Vector charges){
        boolean stobjekts = false;
        for(int i = 0; i < charges.size(); i++)
            stobjekts |= this.addChargeintern((Charge)charges.elementAt(i));
        if(stobjekts){
            this.objectframe.refresh();
            this.objectframe.refreshValues();
            this.applyFieldlines();
        }
    }
    /**Fgt ein Objekt hinzu.
     * @param charge Objekt, das hinzugefgt werden soll
     */
    public void addCharge(Charge charge){
        if(this.addChargeintern(charge)){
            this.objectframe.refresh();
            this.objectframe.refreshValues();
            this.applyFieldlines();
        }
    }
    /**Fgt ein Objekt hinzu.
     * @param charges Objekt, das hinzugefgt werden soll
     * @return false, wenn charge.getClass() == Elektron.class
     */
    private boolean addChargeintern(Charge charge){
        if(charge.getClass() != Elektron.class){
            this.ladungst.add(charge);
            this.objectgroupst.addChild(charge);
            return true;
        }else{
            this.ladungel.add(charge);
            this.objectgroupel.addChild(charge);
            return false;
        }
    }
    /**Lscht alle Objekte, die durch den Vector spezifiziert werden.
     * @param charges Vector mit allen Objekten, die gelscht werden sollen
     */
    public void deleteCharges(Vector charges){
        if(charges != null){
            boolean staticobjekts = false;
            for(int i = 0; i < charges.size(); i++){
                if(((Charge)charges.elementAt(i)).getClass() != Elektron.class){
                    staticobjekts = true;
                    this.ladungst.removeElement(((Charge)charges.elementAt(i)));
                    this.objectgroupst.removeChild((Charge)charges.elementAt(i));
                }else{
                    this.ladungel.removeElement(((Charge)charges.elementAt(i)));
                    this.objectgroupel.removeChild((Charge)charges.elementAt(i));
                }
            }
            if(staticobjekts){
                this.objectframe.refresh();
                this.objectframe.refreshValues();
                this.applyFieldlines();
            }
        }
    }
    /**Lscht ein Objekt.
     * @param charge Objekt, das gelscht werden soll
     */
    public void deleteCharge(Charge charge){
        if(charge.getClass() != Elektron.class){
            this.ladungst.removeElement(charge);
            if(this.selectedobject >= this.ladungst.size())
                this.selectedobject = this.ladungst.size()-1;
            this.objectframe.refresh();
            this.objectframe.refreshValues();
            this.objectgroupst.removeChild(charge);
            this.applyFieldlines();
        }else{
            this.ladungel.removeElement(charge);
            this.objectgroupel.removeChild(charge);
        }
    }
    /**Lscht alle statischen Objekte.
     */
    public void deleteAllStaticCharges(){
        this.objectgroupst.removeAllChildren();
        this.ladungst.removeAllElements();
        this.selectedobject = -1;
        this.objectframe.refresh();
        this.applyFieldlines();
    }
    /**Lscht alle Elektronen.
     */
    public void deleteAllElectrons(){
        try{
            this.objectgroupel.removeAllChildren();
            this.ladungel.removeAllElements();
        }catch(Exception e){}
    }
    /**Methode zum Berechnen der Bewegung der Elektronen.
     * @param stepsize Zeitintervall um das sich die Elektronen bewegen sollen
     * @param precision Anzahl der Teilintervalle in das das angegebene Zeitintervall unterteilt werden soll um die Przision zu erhhen
     */
    public void calculate(double stepsize, int precision){
        Vector ladungenst = (Vector)this.ladungst.clone();
        Vector ladungenel = (Vector)this.ladungel.clone();
        for(int i = 0; i < precision; i++){
            for(int ii = 0; ii < ladungenel.size(); ii++){
                this.deleteCharges(((Charge)ladungenel.elementAt(ii)).prepareMove(ladungenst, stepsize/(double)precision));
            }
            for(int ii = 0; ii < ladungenst.size(); ii++){
                this.deleteCharges(((Charge)ladungenst.elementAt(ii)).prepareMove(ladungenel, stepsize/(double)precision));
            }
            for(int ii = 0; ii < ladungenel.size(); ii++){
                ((Charge)ladungenel.elementAt(ii)).move(stepsize/(double)precision);
            }
        }
        for(int ii = 0; ii < ladungenst.size(); ii++){
            ((Charge)ladungenst.elementAt(ii)).addElectrons();
            ((Charge)ladungenst.elementAt(ii)).refreshTransform();
        }
        for(int ii = 0; ii < ladungenel.size(); ii++){
            ((Charge)ladungenel.elementAt(ii)).refreshTransform();
        }
    }
    /**Setzt alle Beschriftungen aller Fenster.
     * @param language gewnschte Sprache
     * @see #setLabels
     */
    public void setAllLabels(String language){
        this.setLabels(language);
        this.timeframe.setLabels(language);
        this.objectframe.setLabels(language);
        this.efeldmenu.setLabels(language);
    }
    /**Zeichnet die Markierungen fr den Rechenbereich.
     */
    public void applyBoundingBox(){
        if(this.boundingvisible){
            double[] coord0 = {-this.computingarea,this.computingarea,this.computingarea};
            this.boundingbox.setCoordinate(0, coord0);
            double[] coord1 = {this.computingarea,this.computingarea,this.computingarea};
            this.boundingbox.setCoordinate(1, coord1);

            double[] coord2 = {this.computingarea,this.computingarea,this.computingarea};
            this.boundingbox.setCoordinate(2, coord2);
            double[] coord3 = {this.computingarea,-this.computingarea,this.computingarea};
            this.boundingbox.setCoordinate(3, coord3);

            double[] coord4 = {this.computingarea,-this.computingarea,this.computingarea};
            this.boundingbox.setCoordinate(4, coord4);
            double[] coord5 = {-this.computingarea,-this.computingarea,this.computingarea};
            this.boundingbox.setCoordinate(5, coord5);

            double[] coord6 = {-this.computingarea,-this.computingarea,this.computingarea};
            this.boundingbox.setCoordinate(6, coord6);
            double[] coord7 = {-this.computingarea,this.computingarea,this.computingarea};
            this.boundingbox.setCoordinate(7, coord7);


            double[] coord8 = {-this.computingarea,this.computingarea,-this.computingarea};
            this.boundingbox.setCoordinate(8, coord8);
            double[] coord9 = {this.computingarea,this.computingarea,-this.computingarea};
            this.boundingbox.setCoordinate(9, coord9);

            double[] coord10 = {this.computingarea,this.computingarea,-this.computingarea};
            this.boundingbox.setCoordinate(10, coord10);
            double[] coord11 = {this.computingarea,-this.computingarea,-this.computingarea};
            this.boundingbox.setCoordinate(11, coord11);

            double[] coord12 = {this.computingarea,-this.computingarea,-this.computingarea};
            this.boundingbox.setCoordinate(12, coord12);
            double[] coord13 = {-this.computingarea,-this.computingarea,-this.computingarea};
            this.boundingbox.setCoordinate(13, coord13);

            double[] coord14 = {-this.computingarea,-this.computingarea,-this.computingarea};
            this.boundingbox.setCoordinate(14, coord14);
            double[] coord15 = {-this.computingarea,this.computingarea,-this.computingarea};
            this.boundingbox.setCoordinate(15, coord15);


            double[] coord16 = {-this.computingarea,this.computingarea,this.computingarea};
            this.boundingbox.setCoordinate(16, coord16);
            double[] coord17 = {-this.computingarea,this.computingarea,-this.computingarea};
            this.boundingbox.setCoordinate(17, coord17);

            double[] coord18 = {this.computingarea,-this.computingarea,this.computingarea};
            this.boundingbox.setCoordinate(18, coord18);
            double[] coord19 = {this.computingarea,-this.computingarea,-this.computingarea};
            this.boundingbox.setCoordinate(19, coord19);

            double[] coord20 = {this.computingarea,this.computingarea,this.computingarea};
            this.boundingbox.setCoordinate(20, coord20);
            double[] coord21 = {this.computingarea,this.computingarea,-this.computingarea};
            this.boundingbox.setCoordinate(21, coord21);

            double[] coord22 = {-this.computingarea,-this.computingarea,this.computingarea};
            this.boundingbox.setCoordinate(22, coord22);
            double[] coord23 = {-this.computingarea,-this.computingarea,-this.computingarea};
            this.boundingbox.setCoordinate(23, coord23);


            double[] coord24 = {-this.computingarea-10,0,0};
            this.boundingbox.setCoordinate(24, coord24);
            double[] coord25 = {this.computingarea+10,0,0};
            this.boundingbox.setCoordinate(25, coord25);

            double[] coord26 = {0,-this.computingarea-10,0};
            this.boundingbox.setCoordinate(26, coord26);
            double[] coord27 = {0,this.computingarea+10,0};
            this.boundingbox.setCoordinate(27, coord27);

            double[] coord28 = {0,0,-this.computingarea-10};
            this.boundingbox.setCoordinate(28, coord28);
            double[] coord29 = {0,0,this.computingarea+10};
            this.boundingbox.setCoordinate(29, coord29);


            double[] coord30 = {this.computingarea+10,0,0};
            this.boundingbox.setCoordinate(30, coord30);
            double[] coord31 = {this.computingarea+9,0.5,0.5};
            this.boundingbox.setCoordinate(31, coord31);

            double[] coord32 = {this.computingarea+10,0,0};
            this.boundingbox.setCoordinate(32, coord32);
            double[] coord33 = {this.computingarea+9,-0.5,-0.5};
            this.boundingbox.setCoordinate(33, coord33);

            double[] coord34 = {0,this.computingarea+10,0};
            this.boundingbox.setCoordinate(34, coord34);
            double[] coord35 = {0.5,this.computingarea+9,0.5};
            this.boundingbox.setCoordinate(35, coord35);

            double[] coord36 = {0,this.computingarea+10,0};
            this.boundingbox.setCoordinate(36, coord36);
            double[] coord37 = {-0.5,this.computingarea+9,-0.5};
            this.boundingbox.setCoordinate(37, coord37);

            double[] coord38 = {0,0,this.computingarea+10};
            this.boundingbox.setCoordinate(38, coord38);
            double[] coord39 = {0.5,0.5,this.computingarea+9};
            this.boundingbox.setCoordinate(39, coord39);

            double[] coord40 = {0,0,this.computingarea+10};
            this.boundingbox.setCoordinate(40, coord40);
            double[] coord41 = {-0.5,-0.5,this.computingarea+9};
            this.boundingbox.setCoordinate(41, coord41);

            double[] coord42 = {this.computingarea+11,-1,0};
            this.boundingbox.setCoordinate(42, coord42);
            double[] coord43 = {this.computingarea+12,-2,0};
            this.boundingbox.setCoordinate(43, coord43);

            double[] coord44 = {this.computingarea+12,-1,0};
            this.boundingbox.setCoordinate(44, coord44);
            double[] coord45 = {this.computingarea+11,-2,0};
            this.boundingbox.setCoordinate(45, coord45);

            double[] coord46 = {1,this.computingarea+11,0};
            this.boundingbox.setCoordinate(46, coord46);
            double[] coord47 = {1.5,this.computingarea+11.5,0};
            this.boundingbox.setCoordinate(47, coord47);

            double[] coord48 = {1,this.computingarea+12,0};
            this.boundingbox.setCoordinate(48, coord48);
            double[] coord49 = {2,this.computingarea+11,0};
            this.boundingbox.setCoordinate(49, coord49);

            double[] coord50 = {0,0,this.computingarea+11};
            this.boundingbox.setCoordinate(50, coord50);
            double[] coord51 = {0,0,this.computingarea+12};
            this.boundingbox.setCoordinate(51, coord51);

            double[] coord52 = {0,0,this.computingarea+12};
            this.boundingbox.setCoordinate(52, coord52);
            double[] coord53 = {0,-1,this.computingarea+11};
            this.boundingbox.setCoordinate(53, coord53);

            double[] coord54 = {0,-1,this.computingarea+11};
            this.boundingbox.setCoordinate(54, coord54);
            double[] coord55 = {0,-1,this.computingarea+12};
            this.boundingbox.setCoordinate(55, coord55);
        }else{
            double[] coord0 = {0.,0.,0.};
            for(int i = 0; i < 56; i++)
                this.boundingbox.setCoordinate(i, coord0);
        }
        this.applyCenterMark();
    }
    /**Zeichnet den Mittelpunkt.
     */
    public void applyCenterMark(){
        if(this.boundingvisible){
            double[] coord56 = {this.transmitte[0]-0.5,this.transmitte[1]-0.5,this.transmitte[2]-0.5};
            this.boundingbox.setCoordinate(56, coord56);
            double[] coord57 = {this.transmitte[0]+0.5,this.transmitte[1]+0.5,this.transmitte[2]+0.5};
            this.boundingbox.setCoordinate(57, coord57);

            double[] coord58 = {this.transmitte[0]-0.5,this.transmitte[1]-0.5,this.transmitte[2]+0.5};
            this.boundingbox.setCoordinate(58, coord58);
            double[] coord59 = {this.transmitte[0]+0.5,this.transmitte[1]+0.5,this.transmitte[2]-0.5};
            this.boundingbox.setCoordinate(59, coord59);

            double[] coord60 = {this.transmitte[0]-0.5,this.transmitte[1]+0.5,this.transmitte[2]+0.5};
            this.boundingbox.setCoordinate(60, coord60);
            double[] coord61 = {this.transmitte[0]+0.5,this.transmitte[1]-0.5,this.transmitte[2]-0.5};
            this.boundingbox.setCoordinate(61, coord61);

            double[] coord62 = {this.transmitte[0]-0.5,this.transmitte[1]+0.5,this.transmitte[2]-0.5};
            this.boundingbox.setCoordinate(62, coord62);
            double[] coord63 = {this.transmitte[0]+0.5,this.transmitte[1]-0.5,this.transmitte[2]+0.5};
            this.boundingbox.setCoordinate(63, coord63);
        }else{
            double[] coord0 = {0.,0.,0.};
            for(int i = 56; i < 64; i++)
                this.boundingbox.setCoordinate(i, coord0);
        }
    }
    /**Zeichnet die Feldlinien.
     */
    public void applyFieldlines(){
        try{
            this.objectgroupfl.removeAllChildren();
        }catch(Exception e){}
        double Qsum = 0;
        for(int i = 0; i < this.ladungst.size(); i++)
            Qsum += Math.abs(((Charge)this.ladungst.elementAt(i)).Q_As);
        if(this.fieldlinesvisible && (Qsum > 0)){
            Appearance lineappearance = new Appearance();
            lineappearance.setLineAttributes(new LineAttributes(1f, LineAttributes.PATTERN_SOLID, false));
            lineappearance.setColoringAttributes(new ColoringAttributes(1f, 1f, 1f, ColoringAttributes.SHADE_FLAT));
            Vektor3D point = new Vektor3D();
            boolean stop = false;
            double[] linepoints = new double[3];
            for(int i = 0; i < this.ladungst.size(); i++){
                double[][] points = ((Charge)this.ladungst.elementAt(i)).getFieldlineStartingPoints(30);
                if(points != null){
                    for(int ii = 0; ii < points.length; ii++){
                        LineArray lines = new LineArray(this.precision*6*(int)this.computingarea, LineArray.COORDINATES);
                        lines.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
                        stop = false;
                        for(int k = 0; k < lines.getVertexCount(); k+=2){
                            point.x = points[ii][0];
                            point.y = points[ii][1];
                            point.z = points[ii][2];
                            Vektor3D field = new Vektor3D();
                            for(int j = 0; j < this.ladungst.size(); j++){
                                field.addintern(((Charge)ladungst.elementAt(j)).getField(point));
                            }
                            field.scaleintern(Math.signum(((Charge)this.ladungst.elementAt(i)).Q_As)/field.getAbs()/this.precision);
                            points[ii][0] += field.x;
                            points[ii][1] += field.y;
                            points[ii][2] += field.z;
                            linepoints[0] = point.x;
                            linepoints[1] = point.y;
                            linepoints[2] = point.z;
                            lines.setCoordinate(k, linepoints);
                            lines.setCoordinate(k+1, points[ii]);
                            if((Math.abs(points[ii][0]) >= this.computingarea) || (Math.abs(points[ii][1]) >= this.computingarea) || (Math.abs(points[ii][2]) >= this.computingarea))
                                stop = true;
                            else
                                for(int index = 0; index < this.ladungst.size(); index++)
                                    if(((Charge)this.ladungst.elementAt(index)).collided(new Vektor3D(points[ii]), 0, this.precision))
                                        stop = true;
                            if(stop)
                                break;
                        }
                        BranchGroup branch = new BranchGroup();
                        branch.setCapability(BranchGroup.ALLOW_DETACH);
                        branch.addChild(new Shape3D(lines, lineappearance));
                        this.objectgroupfl.addChild(branch);
                    }
                }
            }
        }
    }
    /**Setzt alle Beschriftungen dieses Fensters in der bergebenen Sprache oder in Englisch falls die Sprache nicht untersttzt wird.
     * @param language gewnschte Sprache*/
    public void setLabels(String language){
        String title = "EFeld";
        if(this.autochange)
            title += " (auto change)";
        this.stsave = "Save as";
        this.stload = "Load";
        if((this.country != null) && (this.country.equalsIgnoreCase("DE"))){
            title = "EFeld";
            if(this.autochange)
                title += " (automatischer Wechsel)";
            this.stsave = "Speichern unter";
            this.stload = "Laden";
        }
        this.setTitle(title);
    }
    /**Setzt alle Variablen auf Standardwerte zurck. Ein Aufruf von applyOption() muss folgen damit die Einstellungen auch praktisch umgesetzt werden.*/
    public void setOptionToStandard(){
        try{
            this.country = System.getProperty("user.country");
        }catch(Exception e){}
        if(this.country == null)
            this.country = "US";
        this.hauptfensterbreite = this.getToolkit().getScreenSize().width;
        this.hauptfensterhohe = this.getToolkit().getScreenSize().height-30;
        this.hauptfensterxpos = 0;
        this.hauptfensterypos = 0;
        this.timeframebreite = 250;
        this.timeframehohe = 170;
        this.timeframexpos = this.getToolkit().getScreenSize().width-250;
        this.timeframeypos = this.getToolkit().getScreenSize().height-200;
        this.timeframevisible = true;
        this.objectframebreite = 300;
        this.objectframehohe = 260;
        this.objectframexpos = 0;
        this.objectframeypos = this.getToolkit().getScreenSize().height-290;
        this.objectframevisible = false;
        this.pause = false;
        this.precision = 5;
        this.timestep_nano = 25;
        this.delay = 40;
        this.computingarea = 10.;
        this.boundingvisible = false;
        this.fieldlinesvisible = false;
        this.distance = 20;
        this.viewrot = new Transform3D();
        this.selectedobject = -1;
        transmitte[0] = 0;
        transmitte[1] = 0;
        transmitte[2] = 0;
        this.ortsfaktor.clear();
        this.rotx = 0;
        this.roty = 0;
        this.autochange = false;
        //Two points
        String[] scene1 = {"Settings: precision = 5 timestep = 25.0 gx = 0.0 gy = 0.0 gz = 0.0",
                "Static PointCharge Q = 2.0E-7 x = 3.0 y = 0.0 z = 0.0 a = 1.0 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -1.0E-6 x = -3.0 y = 0.0 z = 0.0 a = 1.0 R = 1.0 G = 1.0 B = 0.0 emit = false"};
        this.scenes[1] = scene1;
        //Quadrupol
        String[] scene2 = {"Settings: precision = 5 timestep = 25.0 gx = 0.0 gy = 0.0 gz = 0.0",
                "Static PointCharge Q = 1.0E-7 x = -3.0 y = -3.0 z = 0.0 a = 1.0 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -1.0E-7 x = -3.0 y = 3.0 z = 0.0 a = 1.0 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -1.0E-7 x = 3.0 y = -3.0 z = 0.0 a = 1.0 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 1.0E-7 x = 3.0 y = 3.0 z = 0.0 a = 1.0 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Electron emitter x = 0.0 y = 0.0 z = -10.0 a = 0.5 b = 0.1 R = 1.0 G = 1.0 B = 0.0 emit = true fx = 0.0 fy = -1.0 fz = 0.0 ux = 0.0 uy = 0.0 uz = 1.0"};
        this.scenes[2] = scene2;
        //Paulfalle
        String[] scene3 = {"Settings: precision = 20 timestep = 25.0 gx = 0.0 gy = 0.0 gz = 0.0",
                "Static PointCharge Q = -1.0E-7 x = 4.0 y = 0.0 z = 0.0 a = 1.0 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -1.0E-7 x = -4.0 y = 0.0 z = 0.0 a = 1.0 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -1.0E-7 x = 0.0 y = -4.0 z = 0.0 a = 1.0 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -1.0E-7 x = 0.0 y = 4.0 z = 0.0 a = 1.0 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Electron emitter x = 0.0 y = 0.0 z = -10.0 a = 1.0 b = 0.15 R = 1.0 G = 1.0 B = 0.0 emit = true fx = 0.0 fy = -1.0 fz = 0.0 ux = 0.0 uy = 0.0 uz = 1.0"};
        this.scenes[3] = scene3;
        //Helix
        String[] scene4 = {"Settings: precision = 5 timestep = 25.0 gx = 0.0 gy = 0.0 gz = 0.0",
                "Static LineCharge Q = 1.0E-7 x = 0.0 y = 0.0 z = 0.0 a = 0.2 b = 20.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Electron emitter x = -1.0 y = -10.0 z = 0.0 a = 0.01 b = 0.05 R = 1.0 G = 1.0 B = 0.0 emit = true fx = 0.0 fy = 0.7071067811865476 fz = -0.7071067811865475 ux = 0.0 uy = 0.7071067811865475 uz = 0.7071067811865476",
                "Electron emitter x = 1.0 y = -10.0 z = 0.0 a = 0.01 b = 0.05 R = 1.0 G = 1.0 B = 0.0 emit = true fx = 0.0 fy = -0.7071067811865476 fz = -0.7071067811865475 ux = 0.0 uy = 0.7071067811865475 uz = -0.7071067811865476",
                "Electron emitter x = 0.0 y = -10.0 z = 1.0 a = 0.01 b = 0.05 R = 1.0 G = 1.0 B = 0.0 emit = true fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.7071067811865476 uy = 0.7071067811865475 uz = 0.0",
                "Electron emitter x = 0.0 y = -10.0 z = -1.0 a = 0.01 b = 0.05 R = 1.0 G = 1.0 B = 0.0 emit = true fx = 0.0 fy = 0.0 fz = -1.0 ux = -0.7071067811865476 uy = 0.7071067811865475 uz = 0.0"};
        this.scenes[4] = scene4;
        //Kondensator
        String[] scene5 = {"Settings: precision = 5 timestep = 25.0 gx = 0.0 gy = 0.0 gz = 0.0",
                "Static PlateCharge Q = 4.0E-9 x = 0.0 y = 0.0 z = 3.0 a = 10.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Static PlateCharge Q = -4.0E-9 x = 0.0 y = 0.0 z = -3.0 a = 10.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Electron emitter x = -10.0 y = 0.0 z = -2.0 a = 0.5 b = 0.1 R = 1.0 G = 1.0 B = 0.0 emit = true fx = 0.0 fy = 0.0 fz = -1.0 ux = 1.0 uy = -1.1102230246251565E-16 uz = 0.0"};
        this.scenes[5] = scene5;
        //Flocke
        String[] scene6 = {"Settings: precision = 5 timestep = 25.0 gx = 0.0 gy = 0.0 gz = 0.0",
                "Static PointCharge Q = -1.0E-7 x = -3.0 y = 0.0 z = 0.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -1.0E-7 x = 3.0 y = 0.0 z = 0.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -1.0E-7 x = 0.0 y = 3.0 z = 0.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -1.0E-7 x = 0.0 y = -3.0 z = 0.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -1.0E-7 x = 0.0 y = 0.0 z = -3.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -1.0E-7 x = 0.0 y = 0.0 z = 3.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 1.0E-6 x = 0.0 y = 0.0 z = 0.0 a = 1.0 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 2.0E-8 x = -3.0 y = -3.0 z = 3.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 2.0E-8 x = 3.0 y = -3.0 z = 3.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 2.0E-8 x = 3.0 y = 3.0 z = -3.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 2.0E-8 x = 3.0 y = -3.0 z = -3.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 2.0E-8 x = 3.0 y = 3.0 z = 3.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 2.0E-8 x = -3.0 y = 3.0 z = -3.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 2.0E-8 x = -3.0 y = -3.0 z = -3.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 2.0E-8 x = -3.0 y = 3.0 z = 3.0 a = 0.5 R = 1.0 G = 1.0 B = 0.0 emit = false"};
        this.scenes[6] = scene6;
        //Lines
        String[] scene7 = {"Settings: precision = 5 timestep = 25.0 gx = 0.0 gy = 0.0 gz = 0.0",
                "Static LineCharge Q = -1.0E-7 x = 3.0 y = 0.0 z = 0.0 a = 0.2 b = 10.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Static LineCharge Q = -1.0E-7 x = -3.0 y = 0.0 z = 0.0 a = 0.2 b = 10.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Static LineCharge Q = -1.0E-7 x = 0.0 y = 0.0 z = -3.0 a = 0.2 b = 10.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Static LineCharge Q = -1.0E-7 x = 0.0 y = 0.0 z = 3.0 a = 0.2 b = 10.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Static LineCharge Q = 5.0E-7 x = 0.0 y = 0.0 z = 0.0 a = 0.2 b = 20.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Static LineCharge Q = 1.0E-7 x = 3.0 y = 0.0 z = -3.0 a = 0.2 b = 3.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Static LineCharge Q = 1.0E-7 x = 3.0 y = 0.0 z = 3.0 a = 0.2 b = 3.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Static LineCharge Q = 1.0E-7 x = -3.0 y = 0.0 z = 3.0 a = 0.2 b = 3.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Static LineCharge Q = 1.0E-7 x = -3.0 y = 0.0 z = -3.0 a = 0.2 b = 3.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0"};
        this.scenes[7] = scene7;
        //Titanic
        String[] scene8 = {"Settings: precision = 5 timestep = 25.0 gx = 0.0 gy = 0.0 gz = 0.0",
                "Static PointCharge Q = 1.0E-6 x = 0.0 y = 0.0 z = 0.0 a = 1.0 R = 1.0 G = 1.0 B = 0.0 emit = false",
                "Static PlateCharge Q = -1.0E-7 x = 0.0 y = 0.0 z = -5.0 a = 10.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Static LineCharge Q = -1.0E-7 x = -5.0 y = 0.0 z = 0.0 a = 0.2 b = 10.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Static LineCharge Q = -1.0E-7 x = 5.0 y = 0.0 z = 0.0 a = 0.2 b = 10.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = 0.0 uy = 1.0 uz = 0.0",
                "Static LineCharge Q = -1.0E-7 x = 0.0 y = 0.0 z = 4.0 a = 0.2 b = 10.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = -1.0 uy = 0.0 uz = 0.0",
                "Static LineCharge Q = 1.0E-7 x = 0.0 y = 0.0 z = -7.0 a = 0.2 b = 10.0 R = 0.0 G = 1.0 B = 0.0 emit = false fx = 0.0 fy = 0.0 fz = -1.0 ux = -0.7071067811865475 uy = -0.7071067811865477 uz = 0.0"};
        this.scenes[8] = scene8;
        //20 Points
        String[] scene9 = {"Settings: precision = 5 timestep = 25.0 gx = 0.0 gy = 0.0 gz = 0.0",
                "Static PointCharge Q = -7.0E-7 x = 0.2284767185932548 y = 4.1330664696810135 z = 3.1340349211382925 a = 0.7 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -6.0E-7 x = -1.2557636663941563 y = -2.9160597772701644 z = 2.388466644189295 a = 0.6 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -4.0E-7 x = 2.5426667582149456 y = 3.67067361266456 z = 2.9993491784440973 a = 0.4 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -9.0E-7 x = -1.5219743586292855 y = 2.9078124418396385 z = 4.895640187341577 a = 0.9 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -9.0E-7 x = 2.558887295541287 y = 3.249336131029537 z = -1.625817589538637 a = 0.9 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -3.0E-7 x = -3.1936427628721775 y = -3.3980417026143037 z = 1.4198426127615784 a = 0.3 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -1.0E-6 x = 2.2284746782856963 y = 1.9846607901323488 z = 4.240627704065512 a = 1.0 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -2.0E-7 x = 0.14988051999521712 y = -4.031441998549704 z = 3.3789193466349374 a = 0.2 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -6.0E-7 x = -4.916909879124567 y = 1.494072956215538 z = 1.3634906661576114 a = 0.6 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = -5.0E-7 x = 1.0874772285858212 y = 1.4378045440354228 z = 4.282980049895675 a = 0.5 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 8.0E-7 x = 0.1343405101324171 y = -1.6106518728362107 z = -1.1099863026436974 a = 0.8 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 8.0E-7 x = -4.618810809850235 y = -2.501270261275781 z = 2.8861864113306233 a = 0.8 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 1.0E-6 x = 4.455652382500169 y = 4.967998268974215 z = -3.108380456241313 a = 1.0 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 6.0E-7 x = -2.140822352472438 y = 0.7859154570536315 z = 4.2688398834748025 a = 0.6 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 1.0E-6 x = -4.577042503465648 y = 0.6425718333498898 z = -4.550768744050647 a = 1.0 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 2.0E-7 x = -0.6167226998761617 y = -1.388851724918613 z = -3.2986725223211675 a = 0.2 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 5.0E-7 x = -3.909243789768828 y = -0.8530687135383204 z = -0.6162634004628007 a = 0.5 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 1.0E-6 x = -0.37229251269377794 y = 4.22908547863813 z = 4.641492265644157 a = 1.0 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 3.0E-7 x = -0.3198825405598562 y = -0.9414285798701236 z = 0.2331544189282191 a = 0.3 R = 0.0 G = 1.0 B = 0.0 emit = false",
                "Static PointCharge Q = 3.0E-7 x = 2.895503426921686 y = -2.3740082316650133 z = 0.7169878213942997 a = 0.3 R = 0.0 G = 1.0 B = 0.0 emit = false"};
        this.scenes[9] = scene9;
        String[] scene0 = {"Settings: precision = 5 timestep = 25.0 gx = 0.0 gy = 0.0 gz = 0.0"};
        this.scenes[0] = scene0;
    }
    /**Ldt Option aus einer ini-Datei und greift auf Standardwerte zurck, wenn Fehler beim Lesen der Werte aus der Datei auftreten, diese gar nicht vorhanden ist oder Parameter fehlen.*/
    public void loadOption(){
        this.setOptionToStandard();
        try{
            RandomAccessFile rafini = new RandomAccessFile("Option\\"+System.getProperty("user.name")+".ini", "r");
            String zeile = "";
            while((zeile = rafini.readLine()) != null){
                if(!zeile.startsWith("//")){
                    if(zeile.startsWith("country = "))
                        this.country = zeile.substring(10);

                    else if(zeile.startsWith("main window width = "))
                        try{
                            this.hauptfensterbreite = Integer.parseInt(zeile.substring(20));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter main window width because of "+nfe.toString());}
                    else if(zeile.startsWith("main window height = "))
                        try{
                            this.hauptfensterhohe = Integer.parseInt(zeile.substring(21));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter main window height because of "+nfe.toString());}
                    else if(zeile.startsWith("main window x-position = "))
                        try{
                            this.hauptfensterxpos = Integer.parseInt(zeile.substring(25));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter main window x-position because of "+nfe.toString());}
                    else if(zeile.startsWith("main window y-position = "))
                        try{
                            this.hauptfensterypos = Integer.parseInt(zeile.substring(25));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter main window y-position because of "+nfe.toString());}

                    else if(zeile.startsWith("time window width = "))
                        try{
                            this.timeframebreite = Integer.parseInt(zeile.substring(20));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter time window width because of "+nfe.toString());}
                    else if(zeile.startsWith("time window height = "))
                        try{
                            this.timeframehohe = Integer.parseInt(zeile.substring(21));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter time window height because of "+nfe.toString());}
                    else if(zeile.startsWith("time window x-position = "))
                        try{
                            this.timeframexpos = Integer.parseInt(zeile.substring(25));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter time window x-position because of "+nfe.toString());}
                    else if(zeile.startsWith("time window y-position = "))
                        try{
                            this.timeframeypos = Integer.parseInt(zeile.substring(25));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter time window y-position because of "+nfe.toString());}
                    else if(zeile.startsWith("display time window = "))
                        try{
                            this.timeframevisible = Boolean.parseBoolean(zeile.substring(22));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter display time window because of "+nfe.toString());}

                    else if(zeile.startsWith("object manager width = "))
                        try{
                            this.objectframebreite = Integer.parseInt(zeile.substring(23));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter object manager width because of "+nfe.toString());}
                    else if(zeile.startsWith("object manager height = "))
                        try{
                            this.objectframehohe = Integer.parseInt(zeile.substring(24));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter object manager height because of "+nfe.toString());}
                    else if(zeile.startsWith("object manager x-position = "))
                        try{
                            this.objectframexpos = Integer.parseInt(zeile.substring(28));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter object manager x-position because of "+nfe.toString());}
                    else if(zeile.startsWith("object manager y-position = "))
                        try{
                            this.objectframeypos = Integer.parseInt(zeile.substring(28));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter object manager y-position because of "+nfe.toString());}
                    else if(zeile.startsWith("display object manager = "))
                        try{
                            this.objectframevisible = Boolean.parseBoolean(zeile.substring(25));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter display object manager because of "+nfe.toString());}

                    else if(zeile.startsWith("break = "))
                        try{
                            this.pause = Boolean.parseBoolean(zeile.substring(8));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter break because of "+nfe.toString());}
                    else if(zeile.startsWith("display bounds = "))
                        try{
                            this.boundingvisible = Boolean.parseBoolean(zeile.substring(17));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter display bounds because of "+nfe.toString());}
                    else if(zeile.startsWith("display field lines = "))
                        try{
                            this.fieldlinesvisible = Boolean.parseBoolean(zeile.substring(22));
                        }catch(NumberFormatException nfe){System.out.println("Ignore parameter display field lines because of "+nfe.toString());}
                }
            }
            rafini.close();
        }catch(Exception e){System.out.println("Use default option because of "+e.toString());}
    }
    /**Setzt die Option fr alle Fenster praktisch um.
     * @see #applyOption
     */
    public void applyAllOption(){
        this.applyOption();
        this.timeframe.applyOption();
        this.objectframe.applyOption();
        this.efeldmenu.applyOption();
    }
    /**Setzt die Option fr dieses Fenster praktisch um (inklsive setLabels()).*/
    public void applyOption(){
        this.setSize(this.hauptfensterbreite, this.hauptfensterhohe);
        this.setLocation(this.hauptfensterxpos, this.hauptfensterypos);
        this.setLabels(this.country);
        for(int i = 0; i < this.ladungst.size(); i++)
            ((Charge)this.ladungst.elementAt(i)).setScale(new Vector3d(((Charge)this.ladungst.elementAt(i)).breite, ((Charge)this.ladungst.elementAt(i)).hohe, ((Charge)this.ladungst.elementAt(i)).tiefe));
        this.applyBoundingBox();
        this.applyFieldlines();
        this.applyView();
    }
    /**Aktualisiert die Perspektive.
     */
    public void applyView(){
        try{
            Transform3D transform = new Transform3D();
            transform.set(new Vector3d(0.,0.,(double)this.distance));
            this.viewtransdist.setTransform(transform);
            this.viewtransrot.setTransform(this.viewrot);
            this.applyViewCenter();
//            this.refresh();
        }catch(Exception e){}
    }
    /**Aktualisiert den Mittelpunklt der Kammera.
     */
    public void applyViewCenter(){
        Transform3D trans3mitte = new Transform3D();
        trans3mitte.set(new Vector3d(this.transmitte[0], this.transmitte[1], this.transmitte[2]));
        this.viewtransmitte.setTransform(trans3mitte);
        this.applyCenterMark();
//        this.refresh();
    }
    /**Speichert die aktuelle Szene.
     * @param path Pfad der Datei in die gespeichert werden soll. Diese wird beui Bedarf erzeugt oder eine gleichnamige gelscht.
     */
    public void saveScene(String path){
        boolean pausetmp = this.pause;
        this.pause = true;
        File file = new File(path);
        if(file.exists())
            file.delete();
        try{
            RandomAccessFile raf = new RandomAccessFile(path, "rw");
            raf.writeBytes("This file contains a scene for the program EFeld.\r\n");
            raf.writeBytes("Diese Datei beinhaltet ein Szenario fr das Programm EFeld.\r\n\r\n");
            String[] scene = this.saveScene();
            for(int i = 0; i < scene.length; i++){
                raf.writeBytes(scene[i]+"\r\n");
            }
            raf.close();
        }catch(Exception e){
            System.out.println("Attention! Can't save scene because of "+e.toString());
        }
        this.pause = pausetmp;
    }
    /**Gibt alle Informationen zurck, die notwendig sind um die Szene zu speichern.
     * @return alle zur Speicherung bestimmten Daten
     */
    private String[] saveScene(){
        boolean pausetmp = this.pause;
        this.pause = true;
        String[] scene = new String[this.ladungst.size()+this.ladungel.size()+1];
        scene[0] = "Settings: precision = "+this.precision+" timestep = "+this.timestep_nano+" gx = "+this.ortsfaktor.x+" gy = "+this.ortsfaktor.y+" gz = "+this.ortsfaktor.z;
        for(int i = 0; i < this.ladungst.size(); i++){
            scene[i+1] = ((Charge)this.ladungst.elementAt(i)).getSaveString();
        }
        for(int i = 0; i < this.ladungel.size(); i++){
            scene[this.ladungst.size()+i+1] = ((Charge)this.ladungel.elementAt(i)).getSaveString();
        }
        this.pause = pausetmp;
        return scene;
    }
    /**Ld ein Szenario aus einer Datei.
     * @param path Pfad der zu ladenden Datei
     * @param deleteold gibt an, ob die vorherige Szene gelscht werden soll (true) oder die neuen Objekte hinzugefgt werden sollen (false)
     */
    public void loadScene(String path, boolean deleteold){
        boolean pausetmp = this.pause;
        this.pause = true;
        boolean fieldlinestmp = this.fieldlinesvisible;
        this.fieldlinesvisible = false;
        File file = new File(path);
        if(file.exists()){
            if(deleteold){
                this.deleteAllStaticCharges();
                this.deleteAllElectrons();
            }
            try{
                RandomAccessFile raf = new RandomAccessFile(path, "r");
                String line = raf.readLine();
                Vector objekte = new Vector();
                while(line != null){
                    if(line.startsWith("Static PointCharge"))
                        objekte.add(new PointCharge(this, line));
                    else if(line.startsWith("Electron emitter"))
                        objekte.add(new EEmitter(this, line));
                    else if(line.startsWith("Electron"))
                        objekte.add(new Elektron(this, line));
                    else if(line.startsWith("Static LineCharge"))
                        objekte.add(new LineCharge(this, line));
                    else if(line.startsWith("Static PlateCharge"))
                        objekte.add(new PlateCharge(this, line));
                    else if(line.startsWith("Settings:")){
                        if(Converter.parseInt(line, 0, "precision = ", " ", 5) > 0)
                            this.precision = Converter.parseInt(line, 0, "precision = ", " ", 5);
                        else
                            this.precision = 5;
                        this.timestep_nano = Converter.parseDouble(line, 0, "timestep = ", " ", 25.);
                        this.ortsfaktor.x = Converter.parseDouble(line, 0, "gx = ", " ", 0.);
                        this.ortsfaktor.y = Converter.parseDouble(line, 0, "gy = ", " ", 0.);
                        this.ortsfaktor.z = Converter.parseDouble(line, 0, "gz = ", " ", 0.);
                        this.timeframe.refreshValues();
                    }
                    line = raf.readLine();
                }
                this.addCharges(objekte);
                raf.close();
            }catch(Exception e){
                System.out.println("Attention! Can't load scene because of "+e.toString());
            }
        }
        this.pause = pausetmp;
        this.fieldlinesvisible = fieldlinestmp;
        this.applyFieldlines();
    }
    /**Ld ein Szenario.
     * @param objekts Array aller Objekte der neuen Szene
     * @param deleteold gibt an, ob die vorherige Szene gelscht werden soll (true) oder die neuen Objekte hinzugefgt werden sollen (false)
     */
    public void loadScene(String[] objekts, boolean deleteold){
        boolean pausetmp = this.pause;
        this.pause = true;
        boolean fieldlinestmp = this.fieldlinesvisible;
        this.fieldlinesvisible = false;
        if(deleteold){
            this.deleteAllStaticCharges();
            this.deleteAllElectrons();
        }
        Vector objekte = new Vector(objekts.length);
        for(int i = 0; i < objekts.length; i++){
            if(objekts[i].startsWith("Static PointCharge"))
                objekte.add(new PointCharge(this, objekts[i]));
            else if(objekts[i].startsWith("Electron emitter"))
                objekte.add(new EEmitter(this, objekts[i]));
            else if(objekts[i].startsWith("Electron"))
                objekte.add(new Elektron(this, objekts[i]));
            else if(objekts[i].startsWith("Static LineCharge"))
                objekte.add(new LineCharge(this, objekts[i]));
            else if(objekts[i].startsWith("Static PlateCharge"))
                objekte.add(new PlateCharge(this, objekts[i]));
            else if(objekts[i].startsWith("Settings:")){
                if(Converter.parseInt(objekts[i], 0, "precision = ", " ", 5) > 0)
                    this.precision = Converter.parseInt(objekts[i], 0, "precision = ", " ", 5);
                else
                    this.precision = 5;
                this.timestep_nano = Converter.parseDouble(objekts[i], 0, "timestep = ", " ", 25.);
                this.ortsfaktor.x = Converter.parseDouble(objekts[i], 0, "gx = ", " ", 0.);
                this.ortsfaktor.y = Converter.parseDouble(objekts[i], 0, "gy = ", " ", 0.);
                this.ortsfaktor.z = Converter.parseDouble(objekts[i], 0, "gz = ", " ", 0.);
                this.timeframe.refreshValues();
            }
        }
        this.addCharges(objekte);
        this.pause = pausetmp;
        this.fieldlinesvisible = fieldlinestmp;
        this.applyFieldlines();
    }
    /**Verarbeitet Drop-Ereignisse auf diesem Fenster
     * @param dtde Geladen wird die Datei, die im DropTargetDropEvent durch den ersten Pfad spezifiziert wird
     */
    public void drop(DropTargetDropEvent dtde){
        String neuerpfad = "";
        dtde.acceptDrop(dtde.getDropAction());
        try{
            neuerpfad += ((File)((java.util.List)dtde.getTransferable().getTransferData(DataFlavor.javaFileListFlavor)).get(0)).getAbsolutePath();
        }catch(Exception e){}
        dtde.dropComplete(true);
        this.loadScene(neuerpfad, true);
    }
    /**leer*/
    public void dragEnter(DropTargetDragEvent dtde){}
    /**leer*/
    public void dragExit(DropTargetEvent dte){}
    /**leer*/
    public void dragOver(DropTargetDragEvent dtde){}
    /**leer*/
    public void dropActionChanged(DropTargetDragEvent dtde){}
    /**Speichert die momentanen Option in der Ini-Datei "Option\\"+System.getProperty("user.name")+".ini". Der Ordner Option wird bei Bedarf erstellt. Alte gleichnamige Ini-Dateien werden berschrieben.*/
    public void saveOption(){
        try{
            File filedir = new File("Option");
            if(!filedir.isDirectory())
                filedir.mkdir();
            File fileini = new File ("Option\\"+System.getProperty("user.name")+".ini");
            if(fileini.exists())
                fileini.delete();
            RandomAccessFile rafini = new RandomAccessFile("Option\\"+System.getProperty("user.name")+".ini", "rw");
            rafini.writeBytes("// This file contents the options of the program EFeld.\r\n");
            rafini.writeBytes("// In dieser Datei sind die Einstellungen des Programms EFeld gespeichert.\r\n");
            rafini.writeBytes("country = "+this.country+"\r\n");

            this.hauptfensterbreite = this.getSize().width;
            rafini.writeBytes("main window width = "+this.hauptfensterbreite+"\r\n");
            this.hauptfensterhohe = this.getSize().height;
            rafini.writeBytes("main window height = "+this.hauptfensterhohe+"\r\n");
            this.hauptfensterxpos = this.getLocationOnScreen().x;
            rafini.writeBytes("main window x-position = "+this.hauptfensterxpos+"\r\n");
            this.hauptfensterypos = this.getLocationOnScreen().y;
            rafini.writeBytes("main window y-position = "+this.hauptfensterypos+"\r\n");

            this.timeframebreite = this.timeframe.getSize().width;
            rafini.writeBytes("time window width = "+this.timeframebreite+"\r\n");
            this.timeframehohe = this.timeframe.getSize().height;
            rafini.writeBytes("time window height = "+this.timeframehohe+"\r\n");
            try{
                this.timeframexpos = this.timeframe.getLocationOnScreen().x;
            }catch(IllegalComponentStateException icse){}
            rafini.writeBytes("time window x-position = "+this.timeframexpos+"\r\n");
            try{
                this.timeframeypos = this.timeframe.getLocationOnScreen().y;
            }catch(IllegalComponentStateException icse){}
            rafini.writeBytes("time window y-position = "+this.timeframeypos+"\r\n");
            rafini.writeBytes("display time window = "+this.timeframevisible+"\r\n");

            this.objectframebreite = this.objectframe.getSize().width;
            rafini.writeBytes("object manager width = "+this.objectframebreite+"\r\n");
            this.objectframehohe = this.objectframe.getSize().height;
            rafini.writeBytes("object manager height = "+this.objectframehohe+"\r\n");
            try{
                this.objectframexpos = this.objectframe.getLocationOnScreen().x;
            }catch(IllegalComponentStateException icse){}
            rafini.writeBytes("object manager x-position = "+this.objectframexpos+"\r\n");
            try{
                this.objectframeypos = this.objectframe.getLocationOnScreen().y;
            }catch(IllegalComponentStateException icse){}
            rafini.writeBytes("object manager y-position = "+this.objectframeypos+"\r\n");
            rafini.writeBytes("display object manager = "+this.objectframevisible+"\r\n");
            rafini.writeBytes("break = "+this.pause+"\r\n");
            rafini.writeBytes("display bounds = "+this.boundingvisible+"\r\n");
            rafini.writeBytes("display field lines = "+this.fieldlinesvisible+"\r\n");
            rafini.close();
        }catch(Exception e){System.out.println("Attention! Can't save option because of "+e.toString());}
    }
    /**Verarbeitet die meisten Eingaben.
     * @see #itemStateChanged
     */
    public void actionPerformed(ActionEvent ae){
        if(ae.getActionCommand().equalsIgnoreCase("Exit"))
            exit();
        else if(ae.getActionCommand().equalsIgnoreCase("Deutsch")){
            this.country = "DE";
            this.setAllLabels("DE");
        }else if(ae.getActionCommand().equalsIgnoreCase("Englisch")){
            this.country = "US";
            this.setAllLabels("US");
        }else if(ae.getActionCommand().equalsIgnoreCase("saveOption")){
            this.saveOption();
        }else if(ae.getActionCommand().equalsIgnoreCase("SaveScene")){
            File file = new File("Scenes");
            if(!file.isDirectory())
                file.mkdir();
            FileDialog dialog = new FileDialog(this, this.stsave, FileDialog.SAVE);
            dialog.setDirectory("Scenes");
            dialog.setFile("*.txt");
            dialog.setVisible(true);
            if(dialog.getFile() != null)
                this.saveScene(dialog.getDirectory()+dialog.getFile());
        }else if(ae.getActionCommand().equalsIgnoreCase("LoadScene")){
            FileDialog dialog = new FileDialog(this, this.stload, FileDialog.LOAD);
            File file = new File("Scenes");
            if(file.isDirectory())
                dialog.setDirectory("Scenes");
            dialog.setFile("*.txt");
            dialog.setVisible(true);
            if(dialog.getFile() != null){
                if(this.autochange){
                    this.autochange = false;
                    this.setLabels(this.country);
                }
                this.loadScene(dialog.getDirectory()+dialog.getFile(), true);
            }
        }else if(ae.getActionCommand().equalsIgnoreCase("AddLoadScene")){
            FileDialog dialog = new FileDialog(this, this.stload, FileDialog.LOAD);
            File file = new File("Scenes");
            if(file.isDirectory())
                dialog.setDirectory("Scenes");
            dialog.setFile("*.txt");
            dialog.setVisible(true);
            if(dialog.getFile() != null){
                if(this.autochange){
                    this.autochange = false;
                    this.setLabels(this.country);
                }
                this.loadScene(dialog.getDirectory()+dialog.getFile(), false);
            }
        }else if(ae.getActionCommand().equalsIgnoreCase("loadOption")){
            this.loadOption();
            this.applyAllOption();
        }else if(ae.getActionCommand().equalsIgnoreCase("clearOption")){
            this.setOptionToStandard();
            this.applyAllOption();
        }else if(ae.getActionCommand().equalsIgnoreCase("Pause")){
            this.pause = !this.pause;
            this.efeldmenu.setPauseLabel(this.country);
            this.timeframe.setPauseLabel(this.country);
        }else if(ae.getActionCommand().equalsIgnoreCase("Reset")){
            this.time_nano = 0;
            this.timeframe.refresh();
        }else if(ae.getActionCommand().equalsIgnoreCase("BoundsIncrease")){
            this.computingarea *= 2;
            for(int i = 0; i < this.ladungst.size(); i++)
                ((Charge)this.ladungst.elementAt(i)).setScale(new Vector3d(((Charge)this.ladungst.elementAt(i)).breite, ((Charge)this.ladungst.elementAt(i)).hohe, ((Charge)this.ladungst.elementAt(i)).tiefe));
            this.applyBoundingBox();
            this.applyFieldlines();
//            this.refresh();
        }else if(ae.getActionCommand().equalsIgnoreCase("BoundsDecrease")){
            this.computingarea /= 2;
            for(int i = 0; i < this.ladungst.size(); i++)
                ((Charge)this.ladungst.elementAt(i)).setScale(new Vector3d(((Charge)this.ladungst.elementAt(i)).breite, ((Charge)this.ladungst.elementAt(i)).hohe, ((Charge)this.ladungst.elementAt(i)).tiefe));
            this.applyBoundingBox();
            this.applyFieldlines();
//            this.refresh();
        }else if(ae.getActionCommand().equalsIgnoreCase("About")){
            this.showAbout();
        }else if(ae.getActionCommand().equalsIgnoreCase("Help")){
            this.showHelp();
        }
    }
    /**Zeigt ein Infofenster an.
     */
    private void showAbout(){
        if((this.country != null) && (this.country.equalsIgnoreCase("DE"))){
            String[][] text = {
                {"EFeld",
                    "",
                    "Dieses Programm dient zur dreidimensionalen Simulation elektrischer Felder.",
                    "Es knnen unterschiedliche statische Ladungsverteilungen eingestellt werden und",
                    "die Bewegung von Elektronen in diesen Feldern simuliert werden.",
                    "",
                    "Verfgbare Objekte: Punktladung, geladene endliche Linie, geladene Platte unendlicher Lnge, Elektronenemitter",
                    "",
                    "Oliver Barta - Version 1.01 - 2009",
                    ""},
                {"Features:",
                    "",
                    "Eigenstndige Rotation um x- und y-Achse; Darstellung von Feldlinien; automatischer Szenenwechsel;",
                    "manuelle Rotation im Pausenmodus; Speichern und Laden von Ladungsverteilungen inklusive Elektronen;",
                    "Zeitverlaufsanzeige; Geschwindigkeitsregulierung; 3D-Ortsfaktor; einstellbare Genauigkeit;",
                    "einstellbarer Rechenbereich; Deutsch-Englisch-Umschaltung; Optionspeicherung; Additives Laden;",
                    "Bercksichtigung der Zunahme der Trgheit bei hohen Geschwindigkeiten; nderung von Position, Ausrichtung,",
                    "Ladung und Gre bei allen Objekten mglich; Emittieren von Elektronen von geladenen Objekten;",
                    "Elektronenemitter mit einstellbarer Austrittsgeschwindigkeit der Elektronen; Verschieben der Kammera;",
                    "temporres Speichern und Laden mit Schnelltasten 1-0; Additives Speichern auf Schnelltasten;",
                    "Anpassen der Kammera- bzw. Objektposition auf Objekt- bzw. Kammeraposition; Laden per Drag and Drop",
                    ""},
                {"Hinweis zu Rechenfehlern",
                    "",
                    "Dieses Programm verwendet die Methode der kleinen Schritte.",
                    "Das heit es betrachtet alle Wert fr kleine Zeit- bzw. Lngenintervalle als konstant.",
                    "Dies kann besonders in Grenzfllen (z.B. wenn ein Wert gegen Null luft) zu Fehlern, wie eckigen Feldlinien,",
                    "Kreuzen von Feldlinien und hnlichem fhren. Diese Fehler sind, wenn sie auftreten, im Normalfall so gravierend,",
                    "dass sie als solche leicht zu erkennen sind. Oft knnen sie durch eine Erhhung der Przision minimiert werden.",
                    "Es sollten besonders kleine Objekte (< 0.2) vermieden werden. Bei sehr vielen Feldlinien und einem groen Rechen-",
                    "bereich wird die CPU und der Speicher stark belastet, wodurch es zu Grafikabstrzen kommen kann.",
                    ""}
            };
            new About((Frame)this, "Info", true, text, 640, 28, 1, true);
        }else{
            String[][] text = {{"EFeld",
                    "",
                    "This program simulates three dimensional elektric fields.",
                    "Different distributions of static electric charges can be used and",
                    "the movement of elektrons in these fields can be simulated.",
                    "",
                    "Available objects: point charge, charged finite line, charged plate with infinity length, electron emitter",
                    "",
                    "Oliver Barta - version 1.01 - 2009",
                    ""}};
            new About((Frame)this, "About", true, text, 640, 28, 1, true);
        }
    }
    /**Zeigt ein Hilfefenster an.
     */
    private void showHelp(){
        if((this.country != null) && (this.country.equalsIgnoreCase("DE"))){
            String[][] text = {
                {"Tastatursteuerung",
                    "F1:",
                    "  Diese Hilfe anzeigen",
                    "F2:",
                    "  Ein-/Ausschalten des automatischen Szenenwechsels",
                    "      zwischen den auf den Tasten 1-0 gespeicherten Szenarien nach einer zuflligen Zeit (ca. 1 min)",
                    "F5:",
                    "  Alles aktualisieren",
                    "*:",
                    "  Heranzoomen",
                    "/:",
                    "  Wegzoomen",
                    "x:",
                    "  nchstes Objekt auswhlen",
                    "y:",
                    "  voriges Objekt auswhlen",
                    "Strg + Q:",
                    "  Programm beenden",
                    "Szenarios knnen per Drag&Drop der entsprechenden Datei auf dieses Programm geladen werden.",
                    "Eine Szenariodatei kann als Parameter beim Starten des Programms bergeben werden.",
                    ""},
                {"Objektverschiebung:",
                    "Alt + Hauptfenster aktiv:",
                    "  + Numpad 0:",
                    "      Verschieben des ausgewhlten Objekts in den Ursprung (0/0/0)",
                    "  + Numpad 2:",
                    "      Verschieben des ausgewhlten Objekts um 1 nach unten (neg. y-Richtung)",
                    "  + Numpad 3:",
                    "      Verschieben des ausgewhlten Objekts um 1 nach vorne (pos. z-Richtung)",
                    "  + Numpad 4:",
                    "      Verschieben des ausgewhlten Objekts um 1 nach links (neg. x-Richtung)",
                    "  + Numpad 5:",
                    "      Verschieben des ausgewhlten Objekts an den Kameramittelpunkt",
                    "  + Numpad 6:",
                    "      Verschieben des ausgewhlten Objekts um 1 nach rechts (pos. x-Richtung)",
                    "  + Numpad 7:",
                    "      Verschieben des ausgewhlten Objekts um 1 nach hinten (neg. z-Richtung)",
                    "  + Numpad 8:",
                    "      Verschieben des ausgewhlten Objekts um 1 nach oben (pos. y-Richtung)",
                    ""},
                {"Objektdrehung:",
                    "Strg:",
                    "  + Numpad 0:",
                    "      Drehung des ausgewhlten Objekts zurcksetzen",
                    "  + Numpad 2:",
                    "      Drehung des ausgewhlten Objekts um die x-Achse um +PI/8",
                    "  + Numpad 4:",
                    "      Drehung des ausgewhlten Objekts um die y-Achse um -PI/8",
                    "  + Numpad 5:",
                    "      Drehung des ausgewhlten Objekts an die Kameradrehung anpassen",
                    "  + Numpad 6:",
                    "      Drehung des ausgewhlten Objekts um die y-Achse um +PI/8",
                    "  + Numpad 7:",
                    "      Drehung des ausgewhlten Objekts um die z-Achse um +PI/8",
                    "  + Numpad 8:",
                    "      Drehung des ausgewhlten Objekts um die x-Achse um -PI/8",
                    "  + Numpad 9:",
                    "      Drehung des ausgewhlten Objekts um die z-Achse um -PI/8",
                    ""},
                {"Kameraverschiebung:",
                    "Alt + Strg:",
                    "  + Numpad 0:",
                    "      Verschieben der Kamera in den Ursprung (0/0/0)",
                    "  + Numpad 2:",
                    "      Verschieben der Kamera um 1 nach unten (neg. y-Richtung)",
                    "  + Numpad 3:",
                    "      Verschieben der Kamera um 1 nach vorne (pos. z-Richtung)",
                    "  + Numpad 4:",
                    "      Verschieben der Kamera um 1 nach links (neg. x-Richtung)",
                    "  + Numpad 5:",
                    "      Verschieben der Kamera an die Koordinaten des ausgewhlten Objekts",
                    "  + Numpad 6:",
                    "      Verschieben der Kamera um 1 nach rechts (pos. x-Richtung)",
                    "  + Numpad 7:",
                    "      Verschieben der Kamera um 1 nach hinten (neg. z-Richtung)",
                    "  + Numpad 8:",
                    "      Verschieben der Kamera um 1 nach oben (pos. y-Richtung)",
                    ""},
                {"Kameradrehung:",
                    "Hauptfenster aktiv:",
                    "  + Numpad 0:",
                    "      Drehung der Kamera zurcksetzen",
                    "  + Numpad 2:",
                    "      Drehung der Kamera um die x-Achse um -PI/8",
                    "  + Numpad 4:",
                    "      Drehung der Kamera um die y-Achse um +PI/8",
                    "  + Numpad 5:",
                    "      Drehung der Kamera an die Drehung des ausgewhlten Objekts anpassen",
                    "  + Numpad 6:",
                    "      Drehung der Kamera um die y-Achse um -PI/8",
                    "  + Numpad 7:",
                    "      Drehung der Kamera um die z-Achse um -PI/8",
                    "  + Numpad 8:",
                    "      Drehung der Kamera um die x-Achse um +PI/8",
                    "  + Numpad 9:",
                    "      Drehung der Kamera um die z-Achse um +PI/8",
                    ""},
                {"Automatische Kameradrehung:",
                    "Automatische Drehung erfolgt nur dann, wenn der Pausenmodus deaktiviert ist!",
                    "Hauptfenster aktiv:",
                    "  + Pfeiltaste links:",
                    "      automatische Drehung der Kamera um die y-Achse um +PI/512 erhhen",
                    "  + Pfeiltaste rechts:",
                    "      automatische Drehung der Kamera um die y-Achse um -PI/512 erhhen",
                    "  + Pfeiltaste oben:",
                    "      automatische Drehung der Kamera um die x-Achse um +PI/512 erhhen",
                    "  + Pfeiltaste unten:",
                    "      automatische Drehung der Kamera um die x-Achse um -PI/512 erhhen",
                    "  + Pfeiltaste links + Pfeiltaste rechts:",
                    "      automatische Drehung der Kamera um die y-Achse stoppen",
                    "  + Pfeiltaste oben + Pfeiltaste unten:",
                    "      automatische Drehung der Kamera um die x-Achse stoppen",
                    ""},
                {"Laden vordefinierter oder vorbergehend gespeicherter Szenarien:",
                    "Tasten 1-0:",
                    "  Laden des Szenarios, welches auf entsprechender Taste gespeichert ist",
                    "Alt + Tasten 1-0:",
                    "  additives Laden des entsprechenden Szenarios",
                    "Strg + Tasten 1-0:",
                    "  Speichern des aktuellen Szenarios auf die entsprechende Taste",
                    "Alt + Strg + Tasten 1-0:",
                    "  additives Speichern des aktuellen Szenarios auf die entsprechende Taste",
                    "F2:",
                    "  Ein-/Ausschalten des automatischen Szenenwechsels",
                    "      zwischen den auf den Tasten 1-0 gespeicherten Szenarien nach einer zuflligen Zeit (ca. 1 min)",
                    ""},
                {"Maussteuerung:",
                    "Mausrdchen:",
                    "  Heran-/Wegzoomen",
                    "linke Maustaste + Ziehen der Maus",
                    "  Drehen der Kamera um die x- bzw. y-Achse der Kameraperspektive",
                    "rechte Maustaste + Ziehen der Maus",
                    "  Drehen der Kamera um die z-Achse der Kameraperspektive",
                    "Alt + Strg + linke Maustaste + Ziehen der Maus",
                    "  Verschiben der Kamera entlang der x- bzw. y-Achse der Kameraperspektive",
                    "Alt + Strg + rechte Maustaste + Ziehen der Maus",
                    "  Verschiben der Kamera entlang der z-Achse der Kameraperspektive",
                    "Strg + linke Maustaste + Ziehen der Maus",
                    "  Drehen des ausgewhlten Objekts um die x- bzw. y-Achse der Kameraperspektive",
                    "Strg + rechte Maustaste + Ziehen der Maus",
                    "  Drehen des ausgewhlten Objekts um die z-Achse der Kameraperspektive",
                    "Alt + linke Maustaste + Ziehen der Maus",
                    "  Verschieben des ausgewhlten Objekts entlang der x- bzw. y-Achse der Kameraperspektive",
                    "Alt + rechte Maustaste + Ziehen der Maus",
                    "  Verschieben des ausgewhlten Objekts entlang der z-Achse der Kameraperspektive",
                    ""}
            };
            new About((Frame)this, "Help", true, text, 640, 28, 1, false);
        }else{
            String[][] text = {{"EFeld",
                    "No english help available.",
                    ""}};
            new About((Frame)this, "Help", true, text, 640, 28, 1, false);
        }
    }
    /**Verarbeitet einige Eingaben.
     * @see #actionPerformed
     */
    public void itemStateChanged(ItemEvent ie){
        if(((CheckboxMenuItem)ie.getSource()).getName().equalsIgnoreCase("TimeFrameShow")){
            this.timeframevisible = !this.timeframevisible;
            this.timeframe.setVisible(this.timeframevisible);
        }else if(((CheckboxMenuItem)ie.getSource()).getName().equalsIgnoreCase("ObjectFrameShow")){
            this.objectframevisible = !this.objectframevisible;
            this.objectframe.setVisible(this.objectframevisible);
        }else if(((CheckboxMenuItem)ie.getSource()).getName().equalsIgnoreCase("BoundsVisible")){
            this.boundingvisible = !this.boundingvisible;
            this.applyBoundingBox();
//            this.refresh();
        }else if(((CheckboxMenuItem)ie.getSource()).getName().equalsIgnoreCase("FieldLinesVisible")){
            this.fieldlinesvisible = !this.fieldlinesvisible;
            this.applyFieldlines();
//            this.refresh();
        }
    }
    /**leer*/
    public void keyTyped(KeyEvent ke){}
    /**Verarbeitet alle Tastatureingaben.
     * @param ke KeyEvent, das verarbeitet werden soll
     */
    public void keyPressed(KeyEvent ke){
        if((ke.getKeyCode() == KeyEvent.VK_Q) && ke.isControlDown() && ke.isAltDown())
            System.exit(0);
        else if(ke.getKeyCode() == KeyEvent.VK_F5)
            this.refreshAll();
        else if(ke.getKeyCode() == KeyEvent.VK_MULTIPLY){
            this.distance--;
            this.applyView();
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_DIVIDE){
            this.distance++;
            this.applyView();
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_X){
            if(this.selectedobject >= this.ladungst.size()-1)
                this.selectedobject = 0;
            else
                this.selectedobject++;
            this.objectframe.selectIndex(this.selectedobject);
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_Y){
            if(this.selectedobject <= 0)
                this.selectedobject = this.ladungst.size()-1;
            else
                this.selectedobject--;
            this.objectframe.selectIndex(this.selectedobject);
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_Q) && ke.isControlDown() && !ke.isAltDown()){
            this.exit();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD8) && ke.isAltDown() && !ke.isControlDown() && this.isActive()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).addToPosition(new Vector3d(0,1,0));
                ((Charge)this.ladungst.elementAt(this.selectedobject)).refreshTransform();
                this.objectframe.refreshPositionValues();
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD2) && ke.isAltDown() && !ke.isControlDown() && this.isActive()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).addToPosition(new Vector3d(0,-1,0));
                ((Charge)this.ladungst.elementAt(this.selectedobject)).refreshTransform();
                this.objectframe.refreshPositionValues();
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD4) && ke.isAltDown() && !ke.isControlDown() && this.isActive()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).addToPosition(new Vector3d(-1,0,0));
                ((Charge)this.ladungst.elementAt(this.selectedobject)).refreshTransform();
                this.objectframe.refreshPositionValues();
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD6) && ke.isAltDown() && !ke.isControlDown() && this.isActive()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).addToPosition(new Vector3d(1,0,0));
                ((Charge)this.ladungst.elementAt(this.selectedobject)).refreshTransform();
                this.objectframe.refreshPositionValues();
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD7) && ke.isAltDown() && !ke.isControlDown() && this.isActive()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).addToPosition(new Vector3d(0,0,-1));
                ((Charge)this.ladungst.elementAt(this.selectedobject)).refreshTransform();
                this.objectframe.refreshPositionValues();
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD3) && ke.isAltDown() && !ke.isControlDown() && this.isActive()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).addToPosition(new Vector3d(0,0,1));
                ((Charge)this.ladungst.elementAt(this.selectedobject)).refreshTransform();
                this.objectframe.refreshPositionValues();
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD0) && ke.isAltDown() && !ke.isControlDown() && this.isActive()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).addToPosition(new Vector3d(-((Charge)this.ladungst.elementAt(this.selectedobject)).pos.x,-((Charge)this.ladungst.elementAt(this.selectedobject)).pos.y,-((Charge)this.ladungst.elementAt(this.selectedobject)).pos.z));
                ((Charge)this.ladungst.elementAt(this.selectedobject)).refreshTransform();
                this.objectframe.refreshPositionValues();
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD5) && ke.isAltDown() && !ke.isControlDown() && this.isActive()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).pos.x = this.transmitte[0];
                ((Charge)this.ladungst.elementAt(this.selectedobject)).pos.y = this.transmitte[1];
                ((Charge)this.ladungst.elementAt(this.selectedobject)).pos.z = this.transmitte[2];
                ((Charge)this.ladungst.elementAt(this.selectedobject)).refreshTransform();
                this.objectframe.refreshPositionValues();
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD8) && ke.isControlDown() && !ke.isAltDown()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).rotX(-Math.PI/8);
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD2) && ke.isControlDown() && !ke.isAltDown()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).rotX(Math.PI/8);
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD4) && ke.isControlDown() && !ke.isAltDown()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).rotY(-Math.PI/8);
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD6) && ke.isControlDown() && !ke.isAltDown()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).rotY(Math.PI/8);
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD7) && ke.isControlDown() && !ke.isAltDown()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).rotZ(Math.PI/8);
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD9) && ke.isControlDown() && !ke.isAltDown()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).rotZ(-Math.PI/8);
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD0) && ke.isControlDown() && !ke.isAltDown()){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).clearRotation();
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD5) && ke.isControlDown() && !ke.isAltDown()){
            if(this.selectedobject >= 0){
                Vector3d scale = new Vector3d();
                ((Charge)this.ladungst.elementAt(this.selectedobject)).transformrot.getScale(scale);
                ((Charge)this.ladungst.elementAt(this.selectedobject)).transformrot = new Transform3D(this.viewrot);
                ((Charge)this.ladungst.elementAt(this.selectedobject)).transformrot.setScale(scale);
                this.applyFieldlines();
            }
//            this.refresh();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD8) && ke.isControlDown() && ke.isAltDown()){
            this.transmitte[1]++;
            this.applyViewCenter();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD2) && ke.isControlDown() && ke.isAltDown()){
            this.transmitte[1]--;
            this.applyViewCenter();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD4) && ke.isControlDown() && ke.isAltDown()){
            this.transmitte[0]--;
            this.applyViewCenter();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD6) && ke.isControlDown() && ke.isAltDown()){
            this.transmitte[0]++;
            this.applyViewCenter();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD7) && ke.isControlDown() && ke.isAltDown()){
            this.transmitte[2]--;
            this.applyViewCenter();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD3) && ke.isControlDown() && ke.isAltDown()){
            this.transmitte[2]++;
            this.applyViewCenter();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD0) && ke.isControlDown() && ke.isAltDown()){
            this.transmitte[0] = 0;
            this.transmitte[1] = 0;
            this.transmitte[2] = 0;
            this.applyViewCenter();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD5) && ke.isControlDown() && ke.isAltDown()){
            if(this.selectedobject >= 0){
                this.transmitte[0] = ((Charge)this.ladungst.elementAt(this.selectedobject)).pos.x;
                this.transmitte[1] = ((Charge)this.ladungst.elementAt(this.selectedobject)).pos.y;
                this.transmitte[2] = ((Charge)this.ladungst.elementAt(this.selectedobject)).pos.z;
            }
            this.applyViewCenter();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD5) && !ke.isControlDown() && !ke.isAltDown() && this.isActive()){
            if(this.selectedobject >= 0){
                Transform3D trans = new Transform3D(((Charge)this.ladungst.elementAt(this.selectedobject)).transformrot);
                trans.setScale(1);
                this.viewrot = trans;
            }
            this.applyView();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD0) && !ke.isControlDown() && !ke.isAltDown() && this.isActive()){
            this.viewrot = new Transform3D();
            this.applyView();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD8) && !ke.isControlDown() && !ke.isAltDown() && this.isActive()){
            Transform3D trans = new Transform3D();
            trans.rotX(Math.PI/8);
            this.viewrot.mul(trans);
            this.applyView();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD2) && !ke.isControlDown() && !ke.isAltDown() && this.isActive()){
            Transform3D trans = new Transform3D();
            trans.rotX(-Math.PI/8);
            this.viewrot.mul(trans);
            this.applyView();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD4) && !ke.isControlDown() && !ke.isAltDown() && this.isActive()){
            Transform3D trans = new Transform3D();
            trans.rotY(Math.PI/8);
            this.viewrot.mul(trans);
            this.applyView();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD6) && !ke.isControlDown() && !ke.isAltDown() && this.isActive()){
            Transform3D trans = new Transform3D();
            trans.rotY(-Math.PI/8);
            this.viewrot.mul(trans);
            this.applyView();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD7) && !ke.isControlDown() && !ke.isAltDown() && this.isActive()){
            Transform3D trans = new Transform3D();
            trans.rotZ(-Math.PI/8);
            this.viewrot.mul(trans);
            this.applyView();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_NUMPAD9) && !ke.isControlDown() && !ke.isAltDown() && this.isActive()){
            Transform3D trans = new Transform3D();
            trans.rotZ(Math.PI/8);
            this.viewrot.mul(trans);
            this.applyView();
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_LEFT) && !ke.isControlDown() && !ke.isAltDown() && this.isActive()){
            this.arrowdown[1] = true;
            if(this.arrowdown[3])
                this.roty = 0;
            else{
                this.roty += Math.PI/512;
                if(this.roty > 2*Math.PI)
                    this.roty -= 2*Math.PI;
            }
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_RIGHT) && !ke.isControlDown() && !ke.isAltDown() && this.isActive()){
            this.arrowdown[3] = true;
            if(this.arrowdown[1])
                this.roty = 0;
            else{
                this.roty -= Math.PI/512;
                if(this.roty < -2*Math.PI)
                    this.roty += 2*Math.PI;
            }
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_UP) && !ke.isControlDown() && !ke.isAltDown() && this.isActive()){
            this.arrowdown[0] = true;
            if(this.arrowdown[2])
                this.rotx = 0;
            else{
                this.rotx += Math.PI/512;
                if(this.rotx > 2*Math.PI)
                    this.rotx -= 2*Math.PI;
            }
            ke.consume();
        }else if((ke.getKeyCode() == KeyEvent.VK_DOWN) && !ke.isControlDown() && !ke.isAltDown() && this.isActive()){
            this.arrowdown[2] = true;
            if(this.arrowdown[0])
                this.rotx = 0;
            else{
                this.rotx -= Math.PI/512;
                if(this.rotx < -2*Math.PI)
                    this.rotx += 2*Math.PI;
            }
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_1){
            this.shortcutKey(ke.isAltDown(), ke.isControlDown(), 1);
            if(this.autochange){
                this.autochange = false;
                this.setLabels(this.country);
            }
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_2){
            this.shortcutKey(ke.isAltDown(), ke.isControlDown(), 2);
            if(this.autochange){
                this.autochange = false;
                this.setLabels(this.country);
            }
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_3){
            this.shortcutKey(ke.isAltDown(), ke.isControlDown(), 3);
            if(this.autochange){
                this.autochange = false;
                this.setLabels(this.country);
            }
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_4){
            this.shortcutKey(ke.isAltDown(), ke.isControlDown(), 4);
            if(this.autochange){
                this.autochange = false;
                this.setLabels(this.country);
            }
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_5){
            this.shortcutKey(ke.isAltDown(), ke.isControlDown(), 5);
            if(this.autochange){
                this.autochange = false;
                this.setLabels(this.country);
            }
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_6){
            this.shortcutKey(ke.isAltDown(), ke.isControlDown(), 6);
            if(this.autochange){
                this.autochange = false;
                this.setLabels(this.country);
            }
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_7){
            this.shortcutKey(ke.isAltDown(), ke.isControlDown(), 7);
            if(this.autochange){
                this.autochange = false;
                this.setLabels(this.country);
            }
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_8){
            this.shortcutKey(ke.isAltDown(), ke.isControlDown(), 8);
            if(this.autochange){
                this.autochange = false;
                this.setLabels(this.country);
            }
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_9){
            this.shortcutKey(ke.isAltDown(), ke.isControlDown(), 9);
            if(this.autochange){
                this.autochange = false;
                this.setLabels(this.country);
            }
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_0){
            this.shortcutKey(ke.isAltDown(), ke.isControlDown(), 0);
            if(this.autochange){
                this.autochange = false;
                this.setLabels(this.country);
            }
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_F1){
            this.showHelp();
            ke.consume();
        }else if(ke.getKeyCode() == KeyEvent.VK_F2){
            this.wait = 800+(int)(Math.random()*600);
            this.autochange = !this.autochange;
            this.setLabels(this.country);
            ke.consume();
        }
    }
    /**Ld ein Szenario, das auf den Schnelltasten 1-0 gespeichert ist.
     * @param alt Alt-Taste gedrckt
     * @param control Control-Taste gedrckt
     * @param number Nummer des zu ladenden Szenarios
     */
    private void shortcutKey(boolean alt, boolean control, int number){
        if(!alt && !control)
            this.loadScene(this.scenes[number], true);
        else if(alt && !control)
            this.loadScene(this.scenes[number], false);
        else if(!alt && control)
            this.scenes[number] = this.saveScene();
        else if(alt && control){
            String[] scene = this.saveScene();
            int settingscount = 0;
            for(int i = 0; i < this.scenes[number].length; i++)
                if(this.scenes[number][i].startsWith("Settings:"))
                    settingscount++;
            String[] newscene = new String[this.scenes[number].length-settingscount+scene.length];
            newscene[0] = scene[0];
            int index = 1;
            for(int i = 0; i < this.scenes[number].length; i++)
                if(!this.scenes[number][i].startsWith("Settings:")){
                    newscene[index] = this.scenes[number][i];
                    index++;
                }
            for(int i = 1; i < scene.length; i++)
                newscene[this.scenes[number].length-settingscount+i] = scene[i];
            this.scenes[number] = newscene;
        }
    }
    /**Ntig zur Verarbeitung von Eingaben per Pfeiltasten
     * @param ke KeyEvent, das verarbeitet werden soll
     */
    public void keyReleased(KeyEvent ke){
        if(ke.getKeyCode() == KeyEvent.VK_UP){
            this.arrowdown[0] = false;
            ke.consume();
        }
        if(ke.getKeyCode() == KeyEvent.VK_LEFT){
            this.arrowdown[1] = false;
            ke.consume();
        }
        if(ke.getKeyCode() == KeyEvent.VK_DOWN){
            this.arrowdown[2] = false;
            ke.consume();
        }
        if(ke.getKeyCode() == KeyEvent.VK_RIGHT){
            this.arrowdown[3] = false;
            ke.consume();
        }
    }
    /**Beendet das Programm.*/
    public void exit(){
        this.setVisible(false);
        this.timeframe.setVisible(false);
        this.objectframe.setVisible(false);
        System.exit(0);
    }
    /**Verarbeitet Bewegungen des Mausrdchens zum Zoomen.
     * @param me zu verarbeitendes MouseWheelEvent
     */
    public void mouseWheelMoved(MouseWheelEvent me){
        this.distance += me.getWheelRotation();
        this.applyView();
    }
    /**Verarbeitet Drag-Operationen der Maus zum Bewegen und Drehen.
     * @param me zu verarbeitendes MouseEvent
     */
    public void mouseDragged(MouseEvent me){
        Vector3d move = new Vector3d();
        if((me.getModifiersEx() == MouseEvent.BUTTON1_DOWN_MASK) || (me.getModifiersEx() == (MouseEvent.BUTTON1_DOWN_MASK | MouseEvent.CTRL_DOWN_MASK)) || (me.getModifiersEx() == (MouseEvent.BUTTON1_DOWN_MASK | MouseEvent.ALT_DOWN_MASK)) || (me.getModifiersEx() == (MouseEvent.BUTTON1_DOWN_MASK | MouseEvent.ALT_DOWN_MASK | MouseEvent.CTRL_DOWN_MASK))){
            move.y = (double)(this.dragpos[0]-me.getX())/100;
            move.x = (double)(this.dragpos[1]-me.getY())/100;
        }else if((me.getModifiersEx() == MouseEvent.BUTTON3_DOWN_MASK) || (me.getModifiersEx() == (MouseEvent.BUTTON3_DOWN_MASK | MouseEvent.CTRL_DOWN_MASK)) || (me.getModifiersEx() == (MouseEvent.BUTTON3_DOWN_MASK | MouseEvent.ALT_DOWN_MASK)) || (me.getModifiersEx() == (MouseEvent.BUTTON3_DOWN_MASK | MouseEvent.ALT_DOWN_MASK | MouseEvent.CTRL_DOWN_MASK))){
            int x = this.dragpos[0]-me.getX();
            int y = this.dragpos[1]-me.getY();
            if(me.getX() > ((Component)me.getSource()).getSize().width/2)
                y = -y;
            if(me.getY() < ((Component)me.getSource()).getSize().height/2)
                x = -x;
            move.z = (double)(x+y)/200;
        }
        if((me.getModifiersEx() == (MouseEvent.BUTTON1_DOWN_MASK | MouseEvent.CTRL_DOWN_MASK)) || (me.getModifiersEx() == (MouseEvent.BUTTON3_DOWN_MASK | MouseEvent.CTRL_DOWN_MASK))){
            move.negate();
            this.viewrot.transform(move);
        }
        if((me.getModifiersEx() == (MouseEvent.BUTTON1_DOWN_MASK | MouseEvent.ALT_DOWN_MASK)) || (me.getModifiersEx() == (MouseEvent.BUTTON1_DOWN_MASK | MouseEvent.ALT_DOWN_MASK | MouseEvent.CTRL_DOWN_MASK)) || (me.getModifiersEx() == (MouseEvent.BUTTON3_DOWN_MASK | MouseEvent.ALT_DOWN_MASK)) || (me.getModifiersEx() == (MouseEvent.BUTTON3_DOWN_MASK | MouseEvent.ALT_DOWN_MASK | MouseEvent.CTRL_DOWN_MASK))){
            move.negate();
            move = new Vector3d(move.y, -move.x, move.z);
            this.viewrot.transform(move);
        }
        Transform3D transformtmpx = new Transform3D();
        transformtmpx.rotX(move.x);
        Transform3D transformtmpy = new Transform3D();
        transformtmpy.rotY(move.y);
        Transform3D transformtmpz = new Transform3D();
        transformtmpz.rotZ(move.z);
        Transform3D transformtmp = new Transform3D();
        transformtmp.mul(transformtmpx);
        transformtmp.mul(transformtmpy);
        transformtmp.mul(transformtmpz);
        if((me.getModifiersEx() == MouseEvent.BUTTON1_DOWN_MASK) || (me.getModifiersEx() == MouseEvent.BUTTON3_DOWN_MASK)){
            this.viewrot.mul(transformtmp);
            try{
                this.viewtransrot.setTransform(this.viewrot);
            }catch(Exception e){}
        }else if((me.getModifiersEx() == (MouseEvent.BUTTON1_DOWN_MASK | MouseEvent.CTRL_DOWN_MASK)) || (me.getModifiersEx() == (MouseEvent.BUTTON3_DOWN_MASK | MouseEvent.CTRL_DOWN_MASK))){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).mulTransform(transformtmp);
                this.applyFieldlines();
            }
        }else if((me.getModifiersEx() == (MouseEvent.BUTTON1_DOWN_MASK | MouseEvent.ALT_DOWN_MASK)) || (me.getModifiersEx() == (MouseEvent.BUTTON3_DOWN_MASK | MouseEvent.ALT_DOWN_MASK))){
            if(this.selectedobject >= 0){
                ((Charge)this.ladungst.elementAt(this.selectedobject)).addToPosition(move);
                ((Charge)this.ladungst.elementAt(this.selectedobject)).refreshTransform();
                this.objectframe.refreshPositionValues();
                this.applyFieldlines();
            }
        }else if((me.getModifiersEx() == (MouseEvent.BUTTON1_DOWN_MASK | MouseEvent.ALT_DOWN_MASK | MouseEvent.CTRL_DOWN_MASK)) || (me.getModifiersEx() == (MouseEvent.BUTTON3_DOWN_MASK | MouseEvent.ALT_DOWN_MASK | MouseEvent.CTRL_DOWN_MASK))){
            double faktor = Math.max(0.05, Math.abs((double)this.distance)/20.);
            move.x *= faktor;
            move.y *= faktor;
            move.z *= faktor;
            this.transmitte[0] -= move.x;
            this.transmitte[1] -= move.y;
            this.transmitte[2] -= move.z;
            this.applyViewCenter();
        }
        this.dragpos[0] = me.getX();
        this.dragpos[1] = me.getY();
//        this.refresh();
    }
    /**Ntig zur Verarbeitung der MouseDragged-Operationen
     * @param me zu verarbeitendes MouseEvent
     */
    public void mousePressed(MouseEvent me){
        this.dragpos[0] = me.getX();
        this.dragpos[1] = me.getY();
    }
    /**Wird beim Schlieen des Hauptfensters aufgerufen und ruft exit() auf
     * @see #exit
     * @param we wird nicht ausgewertet
     */
    public void windowClosing(WindowEvent we){
        this.exit();
    }
    /**leer*/
    public void mouseClicked(MouseEvent me){}
    /**leer*/
    public void mouseEntered(MouseEvent me){}
    /**leer*/
    public void mouseExited(MouseEvent me){}
    /**leer*/
    public void mouseReleased(MouseEvent me){}
    /**leer*/
    public void mouseMoved(MouseEvent me){}
    /**leer*/
    public void windowDeactivated(WindowEvent we){}
    /**leer*/
    public void windowActivated(WindowEvent we){}
    /**leer*/
    public void windowDeiconified(WindowEvent we){}
    /**leer*/
    public void windowIconified(WindowEvent we){}
    /**leer*/
    public void windowClosed(WindowEvent we){}
    /**leer*/
    public void windowOpened(WindowEvent we){}
}
