import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
/**
 * @author Oliver Barta
 * @version 1.0
 */
public class Elektron extends Charge{
    /**Radius der dargestellten Elektronen in Metern*/
    public static final double radius = 5E-2;
    private double masse;
    //[m/s]
    private Vektor3D v;
    //[m/s^2]
    private Vektor3D a;
    public Elektron(EFeld efeld, double x, double y, double z){
        super(efeld, -1.6022E-19, x, y, z, radius*2, radius*2, radius*2, new Color3f(0.2f,0.2f,1f), false, null, null);
        this.v = new Vektor3D();
        this.masse = 9.1094E-31;
        this.specification = "Electron";
    }
    public Elektron(EFeld efeld, double x, double y, double z, double vx, double vy, double vz){
        super(efeld, -1.6022E-19, x, y, z, radius*2, radius*2, radius*2, new Color3f(0.2f,0.2f,1f), false, null, null);
        this.v = new Vektor3D(vx, vy, vz);
        this.masse = 9.1094E-31;
        this.specification = "Electron";
    }
    public Elektron(EFeld efeld, String loadString){
        this(efeld, Converter.parseDouble(loadString, 0, "x = ", " ", 0.), Converter.parseDouble(loadString, 0, "y = ", " ", 0.), Converter.parseDouble(loadString, 0, "z = ", " ", 0.), Converter.parseDouble(loadString, 0, "vx = ", " ", 0.), Converter.parseDouble(loadString, 0, "vy = ", " ", 0.), Converter.parseDouble(loadString, 0, "vz = ", " ", 0.));
    }
    /**
     * @return new Vektor3D()
     */
    public Vektor3D getField(Vektor3D v){
        return new Vektor3D();
    }
    public Vector prepareMove(Vector charges, double timestep){
        Vektor3D field = new Vektor3D();
        for(int i = 0; i < charges.size(); i++){
            field.addintern(((Charge)charges.elementAt(i)).getField(this.pos));
        }
        double gamma = 1./Math.sqrt(1.-Math.pow(this.v.getAbs(), 2.)/89875517873681764.);
        this.a = field.scale(this.Q_As).add(this.efeld.ortsfaktor.scale(-this.masse)).scale(1./(gamma*this.masse));
        if((Math.abs(this.pos.x) > this.efeld.computingarea) || (Math.abs(this.pos.y) > this.efeld.computingarea) || (Math.abs(this.pos.z) > this.efeld.computingarea)){
            Vector vector = new Vector(1);
            vector.add(this);
            return vector;
        }else
            return null;
    }
    public void move(double timestep){
        this.pos.addintern(this.v.scale(timestep).add(this.a.scale(0.5*timestep*timestep)));
        this.v.addintern(this.a.scale(timestep));
    }
    public String toString(){
        return super.toString() + "; m = " + this.masse + "kg ; v = ["+this.v.x+"/"+this.v.y+"/"+this.v.z+"] m/s";
    }
    /**
     * @return {true,false,false}
     */
    public boolean[] setScale(Vector3d scale){
        boolean[] suc = super.setScale(new Vector3d(scale.x,scale.x,scale.x));
        suc[1] = false;
        suc[2] = false;
        return suc;
    }
    public BranchGroup createVisibleObjekt(){
        this.setColor(this.color);
        Sphere sphere = new Sphere(0.5f, this.appearance);
        sphere.setUserData(this);
        BranchGroup group = new BranchGroup();
        group.addChild(sphere);
        return group;
    }
    /**
     * @return null
     */
    public double[][] getFieldlineStartingPoints(int count){
        return null;
    }
    /**
     * @return false
     */
    public boolean collided(Vektor3D position, double radius, int precision){
        return false;
    }
    public String getSaveString(){
        return (this.specification+" x = "+this.pos.x+" y = "+this.pos.y+" z = "+this.pos.z+" vx = "+this.v.x+" vy = "+this.v.y+" vz = "+this.v.z);
    }
}
