import java.util.*;
import javax.media.j3d.*;
import com.sun.j3d.utils.geometry.*;
import javax.vecmath.*;
/**
 * @author Oliver Barta
 * @version 1.0
 */
public class PointCharge extends Charge{
    private double q4PiE;
    public PointCharge(EFeld efeld, double QinAs, double x, double y, double z, double breite, Color3f color, boolean emitting){
        super(efeld, QinAs, x, y, z, breite, breite, breite, color, emitting, null, null);
        this.specification = "Static PointCharge";
    }
    public PointCharge(EFeld efeld, String loadString){
        this(efeld, Converter.parseDouble(loadString, 0, "Q = ", " ", 0.), Converter.parseDouble(loadString, 0, "x = ", " ", 0.), Converter.parseDouble(loadString, 0, "y = ", " ", 0.), Converter.parseDouble(loadString, 0, "z = ", " ", 0.), Converter.parseDouble(loadString, 0, "a = ", " ", 1.), new Color3f(Converter.parseFloat(loadString, 0, "R = ", " ", 1f),Converter.parseFloat(loadString, 0, "G = ", " ", 1f),Converter.parseFloat(loadString, 0, "B = ", " ", 0f)),Converter.parseBoolean(loadString, 0, "emit = ", " ", false));
    }
    public boolean setCharge(double QinAs){
        boolean suc = super.setCharge(QinAs);
        this.q4PiE = this.Q_As/(4.*Math.PI*8.8542E-12);
        return suc;
    }
    public void addElectrons(){
        if(this.emitting)
            if(Math.random()*20 < efeld.timestep_nano*this.breite*this.breite*4){
                double winkela = Math.random()*2*Math.PI;
                double winkelb = Math.random()*Math.PI;
                double cosa = Math.cos(winkela);
                double cosb = Math.cos(winkelb);
                double sina = Math.sin(winkela);
                double sinb = Math.sin(winkelb);
                if(Math.random() > 0.5)
                    this.efeld.addCharge(new Elektron(this.efeld, this.pos.x+cosa*cosb*(this.breite/2+Elektron.radius+1E-10), this.pos.y+sinb*(this.breite/2+Elektron.radius+1E-10), this.pos.z+sina*cosb*(this.breite/2+Elektron.radius+1E-10)));
                else
                    this.efeld.addCharge(new Elektron(this.efeld, this.pos.x+cosa*cosb*(this.breite/2+Elektron.radius+1E-10), this.pos.y-sinb*(this.breite/2+Elektron.radius+1E-10), this.pos.z+sina*cosb*(this.breite/2+Elektron.radius+1E-10)));
            }
    }
    public Vektor3D getField(Vektor3D position){
        double xdif = position.x - this.pos.x;
        double ydif = position.y - this.pos.y;
        double zdif = position.z - this.pos.z;
        double r = Math.sqrt(xdif*xdif+ydif*ydif+zdif*zdif);
        double con = this.q4PiE/(r*r*r);
        if(Double.isInfinite(con))
            con = 0;
        return new Vektor3D(con*xdif,con*ydif,con*zdif);
    }
    public Vector prepareMove(Vector elektrons, double timestep){
        Vector delete = null;
        for(int i = 0; i < elektrons.size(); i++){
            if(this.collided(((Charge)elektrons.elementAt(i)).pos, Elektron.radius, 0)){
                if(delete == null)
                    delete = new Vector();
                delete.add(((Charge)elektrons.elementAt(i)));
            }
        }
        return delete;
    }
    /**
     * @return {true,false,false}
     */
    public boolean[] setScale(Vector3d scale){
        boolean[] suc = super.setScale(new Vector3d(scale.x,scale.x,scale.x));
        suc[1] = false;
        suc[2] = false;
        return suc;
    }
    public BranchGroup createVisibleObjekt(){
        this.setColor(this.color);
        Sphere sphere = new Sphere(0.5f, this.appearance);
        BranchGroup group = new BranchGroup();
        group.addChild(sphere);
        return group;
    }
    public double[][] getFieldlineStartingPoints(int count){
        int a = (int)Math.sqrt(2*count);
        int b = (int)Math.sqrt(count/2);
        double[][] points = new double[2*a*b][3];
        for(int i = 0; i < a; i++){
            for(int ii = 0; ii < b; ii++){
                double cosa = Math.cos(2*Math.PI*(double)i/a);
                double cosb = Math.cos(Math.PI*(double)ii/(2*b));
                double sina = Math.sin(2*Math.PI*(double)i/a);
                double sinb = Math.sin(Math.PI*(double)ii/(2*b));
                points[ii+i*b][0] = this.pos.x+cosa*cosb*this.breite/2;
                points[ii+i*b][1] = this.pos.y+sinb*this.breite/2;
                points[ii+i*b][2] = this.pos.z+sina*cosb*this.breite/2;
                points[a*b+ii+i*b][0] = points[ii+i*b][0];
                points[a*b+ii+i*b][1] = this.pos.y-sinb*this.breite/2;
                points[a*b+ii+i*b][2] = points[ii+i*b][2];
            }
        }
        return points;
    }
    public boolean collided(Vektor3D position, double radius, int precision){
        double xdif = position.x - this.pos.x;
        double ydif = position.y - this.pos.y;
        double zdif = position.z - this.pos.z;
        double r = Math.sqrt(xdif*xdif+ydif*ydif+zdif*zdif);
        if(r < radius+this.breite/2)
            return true;
        return false;
    }
    public String getSaveString(){
        return (this.specification+" Q = "+this.Q_As+" x = "+this.pos.x+" y = "+this.pos.y+" z = "+this.pos.z+" a = "+this.breite+" R = "+this.originalcolor.x+" G = "+this.originalcolor.y+" B = "+this.originalcolor.z+" emit = "+this.originalemitting);
    }
}
