import java.awt.*;
import java.awt.event.*;
/**Zeitfenster des Programms EFeld.
 * @author Oliver Barta
 * @version 1.0
 */
public class TimeFrame extends Dialog implements WindowListener, TextListener, ItemListener{
    private EFeld efeld;
    private Button pause;
    private Button reset;
    private Label sekunden;
    private Label msekunden;
    private Label mikrosekunden;
    private Label nanosekunden;
    private Label bildrate;
    private Label bildrate_label;
    private TextField precision;
    private Label precision_label;
    private Label timestep_label;
    private List timestep_list;
    private TextField timestep;
    private TextField gx;
    private TextField gy;
    private TextField gz;
    private int selfcausedevents;
    /**Erzeugt eine neue Instanz.
     * @param efeld zugehriges EFeld Fenster
     */
    public TimeFrame(EFeld efeld){
        super(efeld);
        this.efeld = efeld;
        this.selfcausedevents = -5;
        this.addWindowListener(this);
        this.addKeyListener(this.efeld);
        this.setLayout(new GridLayout(0,1,0,10));
        this.setBackground(SystemColor.control);

        Panel zeit = new Panel(new GridLayout(1, 0, 0, 0));
        this.sekunden = new Label("-");
        this.sekunden.setAlignment(Label.RIGHT);
        zeit.add(this.sekunden);
        zeit.add(new Label("s"));
        this.msekunden = new Label("-");
        this.msekunden.setAlignment(Label.RIGHT);
        zeit.add(this.msekunden);
        zeit.add(new Label("ms"));
        this.mikrosekunden = new Label("-");
        this.mikrosekunden.setAlignment(Label.RIGHT);
        zeit.add(this.mikrosekunden);
        zeit.add(new Label("s"));
        this.nanosekunden = new Label("-");
        this.nanosekunden.setAlignment(Label.RIGHT);
        zeit.add(this.nanosekunden);
        zeit.add(new Label("ns"));
        this.add(zeit);

        Panel bildrate_p = new Panel(new GridLayout(1, 0, 5, 0));
        this.bildrate_label = new Label();
        bildrate_p.add(this.bildrate_label);
        this.bildrate = new Label("-");
        bildrate_p.add(this.bildrate);
        this.precision_label = new Label();
        bildrate_p.add(this.precision_label);
        this.precision = new TextField("-");
        this.precision.setName("Precision");
        this.precision.addTextListener(this);
        this.precision.addKeyListener(this.efeld);
        bildrate_p.add(this.precision);
        this.add(bildrate_p);

        Panel timestep_p = new Panel(new GridLayout(1, 0, 5, 0));
        this.timestep_label = new Label();
        timestep_p.add(this.timestep_label);
        this.timestep = new TextField("-");
        this.timestep.setName("TimeStep");
        this.timestep.addTextListener(this);
        this.timestep.addKeyListener(this.efeld);
        timestep_p.add(this.timestep);
        this.timestep_list = new List();
        this.timestep_list.add("ns");
        this.timestep_list.add("s");
        this.timestep_list.add("ms");
        this.timestep_list.add("s");
        this.timestep_list.select(0);
        this.timestep_list.addItemListener(this);
        this.timestep_list.addKeyListener(this.efeld);
        timestep_p.add(this.timestep_list);
        this.add(timestep_p);

        Panel ort = new Panel(new GridLayout(1, 0, 0, 0));
        ort.add(new Label("g (x,y,z)"));
        this.gx = new TextField();
        this.gx.setName("gx");
        this.gx.addTextListener(this);
        this.gx.addKeyListener(efeld);
        ort.add(this.gx);
        this.gy = new TextField();
        this.gy.setName("gy");
        this.gy.addTextListener(this);
        this.gy.addKeyListener(efeld);
        ort.add(this.gy);
        this.gz = new TextField();
        this.gz.setName("gz");
        this.gz.addTextListener(this);
        this.gz.addKeyListener(efeld);
        ort.add(this.gz);
        this.add(ort);

        Panel buttons = new Panel(new GridLayout(1, 0, 5, 0));
        this.pause = new Button();
        this.pause.setActionCommand("Pause");
        this.pause.addActionListener(this.efeld);
        this.pause.addKeyListener(this.efeld);
        buttons.add(this.pause);
        this.reset = new Button();
        this.reset.setActionCommand("Reset");
        this.reset.addActionListener(this.efeld);
        this.reset.addKeyListener(this.efeld);
        buttons.add(this.reset);
        this.add(buttons);
        this.applyOption();
    }
    /**Aktualisiert die Zeit und Bildrate.
     */
    public void refresh(){
        this.sekunden.setText(""+(int)(this.efeld.time_nano/1E9));
        this.msekunden.setText(""+(int)((this.efeld.time_nano%1E9)/1E6));
        this.mikrosekunden.setText(""+(int)((this.efeld.time_nano%1E6)/1E3));
        this.nanosekunden.setText(""+(int)(this.efeld.time_nano%1E3));
        String bildzahl = ""+(int)(this.efeld.bildzahl);
        if(bildzahl.length() > 4)
            bildzahl = "---";
        this.bildrate.setText(bildzahl+"/s");
    }
    /**Aktualisiert die Przision, Schrittgre und den Ortsfaktor.
     */
    public void refreshValues(){
        this.selfcausedevents += 5;
        int prepos = this.precision.getCaretPosition();
        this.precision.setText(""+this.efeld.precision);
        this.precision.setCaretPosition(prepos);
        prepos = this.timestep.getCaretPosition();
        this.timestep.setText(""+(this.efeld.timestep_nano/Math.pow(1E3,this.timestep_list.getSelectedIndex())));
        this.timestep.setCaretPosition(prepos);
        prepos = this.gx.getCaretPosition();
        this.gx.setText(""+this.efeld.ortsfaktor.x);
        this.gx.setCaretPosition(prepos);
        prepos = this.gy.getCaretPosition();
        this.gy.setText(""+this.efeld.ortsfaktor.y);
        this.gy.setCaretPosition(prepos);
        prepos = this.gz.getCaretPosition();
        this.gz.setText(""+this.efeld.ortsfaktor.z);
        this.gz.setCaretPosition(prepos);
    }
    /**Setzt die Option fr dieses Fenster praktisch um (inklsive setLabels()).*/
    public void applyOption(){
        this.setSize(this.efeld.timeframebreite, this.efeld.timeframehohe);
        this.setLocation(this.efeld.timeframexpos, this.efeld.timeframeypos);
        this.setLabels(this.efeld.country);
        this.setVisible(this.efeld.timeframevisible);
    }
    /**Setzt alle Beschriftungen dieses Fensters in der bergebenen Sprache oder in Englisch falls die Sprache nicht untersttzt wird.
     * @param language gewnschte Sprache
     */
    public void setLabels(String language){
        String title = "Time";
        String bildrate = "Picturecount";
        String precision = "Precision";
        String timestep = "Time step";
        String reset = "Clear time";
        if((language != null) && (language.equalsIgnoreCase("DE"))){
            title = "Zeit";
            bildrate = "Bildrate";
            precision = "Przision";
            timestep = "Schrittgre";
            reset = "Zeit auf 0 setzen";
        }
        this.setTitle(title);
        this.bildrate_label.setText(bildrate);
        this.precision_label.setText(precision);
        this.timestep_label.setText(timestep);
        this.reset.setLabel(reset);
        this.setPauseLabel(language);
    }
    /**Setzt das aktuelle Pauselabel
     * @param language Sprache
     */
    public void setPauseLabel(String language){
        String pause[] = {"Break", "Resume"};
        if((language != null) && (language.equalsIgnoreCase("DE"))){
            pause[0] = "Pause";
            pause[1] = "Weiter";
        }
        if(this.efeld.pause)
            this.pause.setLabel(pause[1]);
        else
            this.pause.setLabel(pause[0]);
    }
    /**Macht das Fenster sichtbar bzw. unsichtbar*/
    public void setVisible(boolean visible){
        if(this.isVisible()){
            this.efeld.timeframexpos = this.getLocationOnScreen().x;
            this.efeld.timeframeypos = this.getLocationOnScreen().y;
        }
        this.efeld.efeldmenu.applyOption();
        this.refresh();
        this.refreshValues();
        super.setVisible(visible);
        if(this.isVisible()){
            this.efeld.timeframexpos = this.getLocationOnScreen().x;
            this.efeld.timeframeypos = this.getLocationOnScreen().y;
        }
    }
    /**Verarbeitet TextEvents aus diesem Fenster
     * @param te zu verarbeitendes Textereignis
     */
    public void textValueChanged(TextEvent te){
        if(this.selfcausedevents <= 0){
            if(((TextField)te.getSource()).getName().equalsIgnoreCase("Precision")){
                int newpre = 1;
                try{
                    this.precision.setBackground(SystemColor.text);
                    newpre = Integer.parseInt(this.precision.getText());
                    if(newpre > 0){
                        this.efeld.precision = newpre;
                        this.efeld.applyFieldlines();
                    }else
                        throw new Exception();
                }catch(Exception e){
                    this.precision.setBackground(new Color(255,0,0));
                }
            }else if(((TextField)te.getSource()).getName().equalsIgnoreCase("TimeStep")){
                double newtimestep = 1;
                try{
                    this.timestep.setBackground(SystemColor.text);
                    newtimestep = Double.parseDouble(this.timestep.getText());
                    this.efeld.timestep_nano = newtimestep*Math.pow(1E3,this.timestep_list.getSelectedIndex());
                }catch(Exception e){
                    this.timestep.setBackground(new Color(255,0,0));
                }
            }else if(((TextField)te.getSource()).getName().equalsIgnoreCase("gx")){
                try{
                    this.gx.setBackground(SystemColor.text);
                    this.efeld.ortsfaktor.x = Double.parseDouble(this.gx.getText());
                }catch(Exception e){
                    this.gx.setBackground(new Color(255,0,0));
                }
            }else if(((TextField)te.getSource()).getName().equalsIgnoreCase("gy")){
                try{
                    this.gy.setBackground(SystemColor.text);
                    this.efeld.ortsfaktor.y = Double.parseDouble(this.gy.getText());
                }catch(Exception e){
                    this.gy.setBackground(new Color(255,0,0));
                }
            }else if(((TextField)te.getSource()).getName().equalsIgnoreCase("gz")){
                try{
                    this.gz.setBackground(SystemColor.text);
                    this.efeld.ortsfaktor.z = Double.parseDouble(this.gz.getText());
                }catch(Exception e){
                    this.gz.setBackground(new Color(255,0,0));
                }
            }
        }else
            this.selfcausedevents--;
    }
    /**Verarbeitet ItemEvents aus diesem Fenster.
     * @param ie zu verarbeitendes ItemEvent
     */
    public void itemStateChanged(ItemEvent ie){
        this.timestep.setText(""+(this.efeld.timestep_nano/Math.pow(1E3,this.timestep_list.getSelectedIndex())));
    }
    /**Macht dieses Fenster unsichtbar*/
    public void windowClosing(WindowEvent we){
        this.efeld.timeframevisible = false;
        this.setVisible(false);
    }
    /**leer*/
    public void windowDeactivated(WindowEvent we){}
    /**leer*/
    public void windowActivated(WindowEvent we){}
    /**leer*/
    public void windowDeiconified(WindowEvent we){}
    /**leer*/
    public void windowIconified(WindowEvent we){}
    /**leer*/
    public void windowClosed(WindowEvent we){}
    /**leer*/
    public void windowOpened(WindowEvent we){}
}
